# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import fcntl
import gtk
import logging
import os
import shutil

from xdg.BaseDirectory import xdg_config_home

from gettext import gettext as _

import config as CONFIG

import covers
import lyrics
import players
import tabs

from core import config

from core.common import DBUS_NAME
from core.common import DBUS_PATH
from core.common import ICON_PATH
from core.common import LOCK_PATH

from core.utils import get_themes

from covers import GenericCover
from lyrics import GenericLyric
from tabs import GenericTab
from widgets.pluginview import PluginView
from widgets.themeview import ThemeView

c = logging.getLogger(__name__)

ELLIPSIZE_VALS = ('cut', 'ellipsize-word', 'ellipsize-char', 'scroll', 'theme')
ELLIPSIZE_OPTIONS = (_('Cut'),
                     _('Ellipsize by word'),
                     _('Ellipsize by character'),
                     _('Scroll'),
                     _('Defined by theme'))

LYRIC_WINDOW_TARGETS = (
    ('new', _('New window')),
    ('separate', _('Separate windows')),
    ('shared', _('Same window'))
)

RIGTH_WIDTH = 400
LEFT_WIDTH = 140
CENTER_WIDTH = 320

def _make_frame(title):
    frame = gtk.Frame()
    label = gtk.Label('<b>' + title + '</b>')
    label.set_use_markup(True)
    frame.set_label_widget(label)
    frame.set_shadow_type(gtk.SHADOW_NONE)
    box = gtk.VBox()
    align = gtk.Alignment(0.0, 0.0, 1.0, 1.0)
    align.set_padding(0, 10, 15, 0)
    align.add(box)
    frame.add(align)
    return frame, box

class Configuration(gtk.Window):

    __instance = None

    config_names = ['general', 'interface', 'theme', 'covers', 'lyrics', 'tabs']
    config_tags = {
        'general' :_('General'),
        'interface' : _('Interface'),
        'theme' : _('Theme'),
        'covers' : _('Covers'),
        'lyrics' : _('Lyrics'),
        'tabs' : _('Tablatures')
    }

    @staticmethod
    def run(cg=None):
        if Configuration.__instance is None:
            if cg:
                cg.update_config()
            Configuration.__instance = Configuration(cg)
            Configuration.__instance.connect('destroy', Configuration.__clean)
        else:
            c.warning(_('Configuration is already running'))
            Configuration.__instance.present()

    @staticmethod
    def __clean(widget):
        Configuration.__instance = None

    def __init__(self, parent = None):

        gtk.Window.__init__(self)

        # now we create some methods to interract with main covergloobus window
        if parent:
            self.quit = self.destroy
            # use CoverGloobus class methods if it is instatiated in current process
            self.parent_init = parent.init
            self.parent_move = parent._window.move
            self.parent_theme = parent._window.init
            self.parent_resize = parent._window.scale
        else:
            self.quit = gtk.main_quit

            try:
                lock = open(LOCK_PATH, 'w')
                fcntl.lockf(lock, fcntl.LOCK_EX | fcntl.LOCK_NB)
                c.debug('CoverGloobus not running, using void')
                # send to void all calls if covergloobus is not running
                def void(*args):
                    pass
                self.parent_init = void
                self.parent_move = void
                self.parent_theme = void
                self.parent_resize = void
                lock.close()
            except:
                c.debug('CoverGloobus running, using dbus interface')
                # extract dbus methods if covergloobus is running
                bus = dbus.SessionBus()
                obj = bus.get_object(DBUS_NAME, DBUS_PATH)
                self.parent_init = obj.get_dbus_method('Init', DBUS_NAME)
                self.parent_move = obj.get_dbus_method('Move', DBUS_NAME)
                self.parent_theme = obj.get_dbus_method('SetTheme', DBUS_NAME)
                self.parent_resize = obj.get_dbus_method('Resize', DBUS_NAME)

        self.src_resize = config.getint('Window', 'resize')
        self.src_theme = config.get('Window', 'theme')

        self.set_title(_('CoverGloobus Configuration'))
        self.set_icon_from_file(ICON_PATH)

        self.set_position(gtk.WIN_POS_CENTER)

        self.notebook = gtk.Notebook()
        self.notebook.set_border_width(5)
        self.notebook.set_tab_pos(gtk.POS_LEFT)

        self.container = gtk.VBox()

        self.make_pages()
        self.make_buttons()

        self.add(self.container)
        self.show_all()

    def make_pages(self):
        self.pages = {}
        for name in self.config_names:
            method = getattr(self, 'make_'+name)
            page = method()
            page.set_size_request(CENTER_WIDTH, -1)
            label = gtk.Label(self.config_tags[name])
            valign = gtk.Alignment(0, 0, 1, 1)
            valign.add(page)
            valign.set_border_width(10)
            self.notebook.append_page(valign, label)
            self.pages[name] = page
        self.container.pack_start(self.notebook)

    def make_buttons(self):
        save = gtk.Button(stock=gtk.STOCK_SAVE)
        save.set_size_request(100,25)
        cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
        cancel.set_size_request(100,25)
        save.connect('clicked', self.save_config)
        cancel.connect("clicked", self.cancel)

        buttons = gtk.HBox()
        buttons.set_border_width(5)
        buttons.set_spacing(10)
        buttons.add(cancel)
        buttons.add(save)

        halign = gtk.Alignment(1, 0, 0, 0)
        halign.add(buttons)
        self.container.pack_end(halign,False)

    def make_general(self):
        page = gtk.VBox()

        #=================== PLAYER ================== #
        frame, frame_box = _make_frame(_('Player'))

        box = gtk.HBox()
        label = gtk.Label(_('Plugin') + ':')
        label.set_alignment(0,0.5)
        label.set_size_request(LEFT_WIDTH, -1)
        self.entry_player = gtk.combo_box_new_text()
        self.entry_player.set_has_tooltip(True)
        plugins = players.get_players()
        current = config.get("General", "player")
        for i in xrange(len(plugins)):
            self.entry_player.append_text(plugins[i])
            if plugins[i] == current:
                self.entry_player.set_active(i)
                c.debug("Active:" + plugins[i])
        self.entry_player.connect("changed", self.player_entry_changed)
        box.pack_start(label,False)
        box.pack_start(self.entry_player)
        frame_box.pack_start(box, False)

        box = gtk.HBox()
        self.player_about = gtk.Button(_('About Plugin'))
        self.player_config = gtk.Button(_('Configure Plugin'))

        self.player_about.connect('clicked', self.on_plugin_about)
        self.player_config.connect('clicked', self.on_plugin_config)

        box.pack_end(self.player_config, False, False)
        box.pack_end(self.player_about, False, False)

        klass = players.get_class(current)
        self.entry_player.set_tooltip_text(klass.__desc__)
        if hasattr(klass, 'configure'):
            self.player_config.set_sensitive(True)
        else:
            self.player_config.set_sensitive(False)

        frame_box.pack_start(box, False)

        page.pack_start(frame, False)

        #=============== DOCKS ================== #
        frame, frame_box = _make_frame(_('Docks'))

        self.check_docky = gtk.CheckButton(label=_("Show cover on docks"), use_underline=False)
        self.check_docky.set_active(config.getboolean("Docky","show"))
        frame_box.pack_start(self.check_docky, False)

        self.check_docky_time = gtk.CheckButton(label=_("Show time on docks"), use_underline=False)
        self.check_docky_time.set_active(config.getboolean("Docky", "time"))
        frame_box.pack_start(self.check_docky_time, False)

        page.pack_start(frame, False)

        #=============== LYRICS ================== #
        frame, frame_box = _make_frame(_('Lyrics/tablatures'))

        self.check_lyrics_refresh = gtk.CheckButton(label=_("Refresh lyrics/tabs when song changes"), use_underline=False)
        self.check_lyrics_refresh.set_active(config.getboolean("Lyrics","refresh"))
        frame_box.pack_start(self.check_lyrics_refresh,False)

        self.check_lyrics_cache = gtk.CheckButton(label=_("Cache lyrics/tabs"), use_underline=False)
        self.check_lyrics_cache.set_active(config.getboolean("Lyrics","cache"))
        frame_box.pack_start(self.check_lyrics_cache,False)

        page.pack_start(frame, False)

        frame, frame_box = _make_frame(_('Miscellaneous'))

        #============= START AT LOGIN ================ #
        self.check_autostart = gtk.CheckButton(label=_("Start at login"), use_underline=False)
        self.check_autostart.set_active(config.getint("General", "start_at_login"))
        frame_box.pack_start(self.check_autostart, False)

        #============= HIDE IF INACTIVE ================ #
        self.check_hide_inactive = gtk.CheckButton(label=_("Hide when player is closed"), use_underline=False)
        self.check_hide_inactive.set_active(config.getint("General", "hide_if_inactive"))
        frame_box.pack_start(self.check_hide_inactive, False)

        #============= SHOW STATUS ICON ================ #
        self.check_status_icon = gtk.CheckButton(label=_("Show status icon in system tray"), use_underline=False)
        self.check_status_icon.set_active(config.getint("General","show_status_icon"))
        frame_box.pack_start(self.check_status_icon, False)

        page.pack_start(frame, False)

        return page

    def make_interface(self):
        page = gtk.VBox()

        frame, frame_box = _make_frame(_('Window'))

        #=============== ALLWAYS ON TOP ================== #
        self.check_ontop = gtk.CheckButton(label=_("Always on top"), use_underline=False)
        self.check_ontop.set_active(config.getboolean("Window","ontop"))
        frame_box.pack_start(self.check_ontop, False)

        #============= STICK TO DESKTOP ================ #
        self.check_stick = gtk.CheckButton(label=_("Stick to desktop"), use_underline=False)
        self.check_stick.set_active(config.getboolean("Window","stick"))
        frame_box.pack_start(self.check_stick, False)

        #================= START POSITION ================= #
        box = gtk.HBox()
        label = gtk.Label(_("Start position:"))
        label.set_alignment(0,0.5)
        label.set_size_request(LEFT_WIDTH, -1)
        self.entry_pos_x = gtk.Entry(5)
        self.entry_pos_x.set_width_chars(5)
        self.entry_pos_x.set_text(config.get('Window','x'))
        self.entry_pos_x.set_tooltip_text(_('X position'))
        self.entry_pos_y = gtk.Entry(5)
        self.entry_pos_y.set_width_chars(5)
        self.entry_pos_y.set_text(config.get('Window','y'))
        self.entry_pos_y.set_tooltip_text(_('Y position'))
        middle = gtk.Label('x')
        box.pack_start(label, False)
        box.pack_start(self.entry_pos_x)
        box.pack_start(middle, False)
        box.pack_start(self.entry_pos_y)
        frame_box.pack_start(box, False)

        #============== WINDOW RESIZE ============== #
        box = gtk.HBox()
        label = gtk.Label(_("Resize window:"))
        label.set_alignment(0,0.5)
        label.set_size_request(LEFT_WIDTH, -1)
        init = config.getint('Window', 'resize')
        adjustment = gtk.Adjustment(init, 1, 500, 5, 1)
        self.spin_resize = gtk.SpinButton(adjustment)
        self.spin_resize.connect('value-changed', self.window_resize)
        box.pack_start(label, False)
        box.pack_start(self.spin_resize)
        frame_box.pack_start(box, False)

        #============= SHOW BACKGROUND ================= #
        self.check_background = gtk.CheckButton(label=_("Show background"), use_underline=False)
        self.check_background.set_active(config.getint("Window", "show_background"))
        frame_box.pack_start(self.check_background, False)

        #============== BACKGROUND COLOR ============== #
        box = gtk.HBox()
        label = gtk.Label(_("Background color:"))
        label.set_alignment(0,0.5)
        label.set_size_request(LEFT_WIDTH, -1)
        self.color = gtk.ColorButton()
        try:
            color = gtk.gdk.color_parse(config.get("Window","background_color"))
            self.color.set_color(color)
            alpha = config.getfloat("Window","background_opacity") * 65535.0
            self.color.set_alpha(int(alpha))
        except:
            pass
        self.color.set_use_alpha(True)

        box.pack_start(label, False)
        box.pack_end(self.color)
        frame_box.pack_start(box, False)

        page.pack_start(frame, False)

        #=================== LYRICS WINDOW TARGET ================== #
        frame, frame_box = _make_frame(_('Lyrics/tablatures'))

        box = gtk.HBox()
        label = gtk.Label(_("Open lyrics/tabs in:"))
        label.set_alignment(0,0.5)
        label.set_size_request(LEFT_WIDTH, -1)
        self.entry_lyric_window = gtk.combo_box_new_text()
        self.entry_lyric_window.set_size_request(200,30)
        current = config.get("Lyrics","window")
        for i in xrange(len(LYRIC_WINDOW_TARGETS)):
            self.entry_lyric_window.append_text(LYRIC_WINDOW_TARGETS[i][1])
            if LYRIC_WINDOW_TARGETS[i][0] == current:
                self.entry_lyric_window.set_active(i)
        box.pack_start(label, False)
        box.pack_start(self.entry_lyric_window)
        frame_box.pack_start(box, False)

        self.check_lyrics_widget = gtk.CheckButton(label=_("Show lyrics/tabs windows as widget"), use_underline=False)
        self.check_lyrics_widget.set_active(config.getboolean("Lyrics","widget"))
        frame_box.pack_start(self.check_lyrics_widget, False)

        page.pack_start(frame, False)

        frame, frame_box = _make_frame(_('Miscellaneous'))

        #=================== LONG TEXT ================== #
        box = gtk.HBox()
        label = gtk.Label(_("Long text:"))
        label.set_alignment(0,0.5)
        label.set_size_request(LEFT_WIDTH, -1)
        self.entry_long_text = gtk.combo_box_new_text()
        #self.entry_long_text.set_has_tooltip(True)
        current = config.get('Window', 'long_text')
        for i in range(len(ELLIPSIZE_VALS)):
            self.entry_long_text.append_text(ELLIPSIZE_OPTIONS[i])
            if current == ELLIPSIZE_VALS[i]:
                self.entry_long_text.set_active(i)
        box.pack_start(label, False)
        box.pack_start(self.entry_long_text)
        frame_box.pack_start(box, False)

        page.pack_start(frame, False)

        return page

    def make_theme(self):
        self.themes = ThemeView()
        self.themes.set_themes(get_themes())
        self.themes.set_selected(self.src_theme)
        self.themes.connect('theme-changed', self.on_theme_changed)
        return self.themes

    def make_covers(self):
        plugins = []
        klasses = {}
        activated = config.get('Covers', 'search_engines').split(',')
        for klass in GenericCover.load():
            klasses[klass.__name__] = klass
        for name in activated:
            if name in klasses:
                klass = klasses.pop(name)
                plugins.append([klass.__title__, klass.__desc__, True, klass])
        for klass in klasses.itervalues():
            plugins.append([klass.__title__, klass.__desc__, False, klass])
        self.covers_view = PluginView()
        self.covers_view.set_plugins(plugins)
        return self.covers_view

    def make_lyrics(self):
        plugins = []
        klasses = {}
        activated = config.get('Lyrics', 'lyrics_source').split(',')
        for klass in GenericLyric.load():
            klasses[klass.__name__] = klass
        for name in activated:
            if name in klasses:
                klass = klasses.pop(name)
                plugins.append([klass.__title__, klass.__desc__, True, klass])
        for klass in klasses.itervalues():
            plugins.append([klass.__title__, klass.__desc__, False, klass])
        self.lyrics_view = PluginView()
        self.lyrics_view.set_plugins(plugins)
        return self.lyrics_view

    def make_tabs(self):
        plugins = []
        klasses = {}
        activated = config.get('Lyrics', 'tabs_source').split(',')
        for klass in GenericTab.load():
            klasses[klass.__name__] = klass
        for name in activated:
            if name in klasses:
                klass = klasses.pop(name)
                plugins.append([klass.__title__, klass.__desc__, True, klass])
        for klass in klasses.itervalues():
            plugins.append([klass.__title__, klass.__desc__, False, klass])
        self.tabs_view = PluginView()
        self.tabs_view.set_plugins(plugins)
        return self.tabs_view

    def on_plugin_about(self, widget):
        klass = players.get_class(self.entry_player.get_active_text())
        dialog = gtk.AboutDialog()
        dialog.set_logo_icon_name("covergloobus")
        dialog.set_name(klass.__title__)
        dialog.set_version(klass.__version__)
        dialog.set_comments(klass.__desc__)

        try:
            dialog.set_authors([a.strip() for a in klass.__authors__.split(',')])
        except:
            pass

        dialog.run()
        dialog.destroy()

    def on_plugin_config(self, widget):
        klass = players.get_class(self.entry_player.get_active_text())
        dialog = gtk.Dialog(_('%s configuration') % klass.__title__, self,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                            gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))

        klass.configure(dialog)
        dialog.destroy()

    # ====================================================================== #
    def on_theme_changed(self, widget, path):
        self.parent_theme(path)
        self.parent_move(int(self.entry_pos_x.get_text()),
                         int(self.entry_pos_y.get_text()))
        self.spin_resize.set_value(100)

    def player_entry_changed(self, widget):
        klass = players.get_class(widget.get_active_text())
        self.entry_player.set_tooltip_text(klass.__desc__)
        if hasattr(klass, 'configure'):
            self.player_config.set_sensitive(True)
        else:
            self.player_config.set_sensitive(False)

    def save_config(self,button):
        c.info(_('Saving new configuration'))
        c.debug("x:" + self.entry_pos_x.get_text())
        c.debug("y:" + self.entry_pos_y.get_text())
#        c.debug("Theme:" + self.entry_theme.get_active_text())
#        c.debug("Player:" + self.entry_player.get_active_text())
#        c.debug("Lyrics:" + self.entry_lyrics.get_active_text())
#        c.debug("Tabs:" + self.entry_tabs.get_active_text())
#        c.debug("Ontop:" + str(int(self.check_ontop.get_active())))
#        c.debug("Docky:" + str(int(self.check_docky.get_active())))
#        c.debug("Stick to desktop:" + str(int(self.check_stick.get_active())))
#        c.debug("Show status icon:" + str(int(self.check_status_icon.get_active())))
#        c.debug("Show background:" + str(int(self.check_background.get_active())))
#        c.debug("Background opacity:" + str(self.spin_opacity.get_value()))

        # General
        config.set('General', 'start_at_login', str(int(self.check_autostart.get_active())))
        config.set('General', 'hide_if_inactive', str(int(self.check_hide_inactive.get_active())))
        config.set('General', 'show_status_icon', str(int(self.check_status_icon.get_active())))
        config.set('General', 'player', self.entry_player.get_active_text())

        # Covers
        sources = [plugin[3].__name__ for plugin in self.covers_view.get_plugins() if plugin[2]]
        config.set('Covers', 'search_engines', ','.join(sources))

        # Docks
        config.set('Docky', 'show', str(int(self.check_docky.get_active())))
        config.set('Docky', 'time', str(int(self.check_docky_time.get_active())))

        sources = [plugin[3].__name__ for plugin in self.lyrics_view.get_plugins() if plugin[2]]
        config.set('Lyrics', 'lyrics_source', ','.join(sources))

        sources = [plugin[3].__name__ for plugin in self.tabs_view.get_plugins() if plugin[2]]
        config.set('Lyrics', 'tabs_source', ','.join(sources))

        config.set('Lyrics', 'window', LYRIC_WINDOW_TARGETS[self.entry_lyric_window.get_active()][0])
        config.set('Lyrics', 'widget', str(int(self.check_lyrics_widget.get_active())))
        config.set('Lyrics', 'refresh', str(int(self.check_lyrics_refresh.get_active())))
        config.set('Lyrics', 'cache', str(int(self.check_lyrics_cache.get_active())))
        #config.set('Lyrics', 'lyricsfly_key', self.entry_lyricsfly_key.get_text())

        # Window
        config.set('Window', 'theme', self.themes.get_selected())
        config.set('Window', 'x', self.entry_pos_x.get_text())
        config.set('Window', 'y', self.entry_pos_y.get_text())
        config.set('Window', 'ontop', str(int(self.check_ontop.get_active())))
        config.set('Window', 'stick', str(int(self.check_stick.get_active())))
        config.set('Window', 'resize', str(int(self.spin_resize.get_value())))
        config.set('Window', 'show_background', str(int(self.check_background.get_active())))
        config.set('Window', 'background_opacity', str(self.color.get_alpha()/65535.0))
        config.set('Window', 'background_color', str(self.color.get_color()))

        config.set('Window', 'long_text', ELLIPSIZE_VALS[self.entry_long_text.get_active()])

        config.save()

        self.set_autostart(self.check_autostart.get_active())

        self.parent_init()
        self.quit()

    def cancel(self, widget=None):
        self.parent_theme(self.src_theme)
        self.parent_resize(self.src_resize)
        self.quit()

    def window_resize(self, widget):
        percent = int(widget.get_value())
        self.parent_resize(percent)

    def set_autostart(self, autostart):
        autostart_dir = os.path.join(xdg_config_home, 'autostart')
        autostart_path = os.path.join(autostart_dir, 'covergloobus.desktop')
        desktop_file = os.path.join(CONFIG.datadir, 'applications', 'covergloobus.desktop')

        if autostart:
            if not os.path.exists(autostart_dir):
                os.mkdir(autostart_dir)
            if os.path.isdir(autostart_dir) and not os.path.exists(autostart_path):
                if hasattr(os, "symlink"):
                    os.symlink(desktop_file, autostart_path)
                else:
                    shutil.copyfile(desktop_file, autostart_path)
        else:
            if os.path.isfile(autostart_path):
                os.remove(autostart_path)
