# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import fcntl
import gobject
import logging
import gtk
import os

from dbus.mainloop.glib import DBusGMainLoop

from gettext import gettext as _

from core import config

from core.common import COVER, STATE, TIME, TRACK, PLAYER
from core.common import LOCK_PATH

from core.coversearch import CoverSearch
from core.dbuswrapper import DBusWrapper
from core.docky import Dock
from core.lyricsearch import LyricSearch
from core.observer import Observer
from core.track import State
from core.track import Track
from core.ui import CoverGloobusUI

from players import get_player

c = logging.getLogger(__name__)

EMPTY_TRACK = Track()

class CoverGloobus(object):
    covers = None
    dbus = None
    dock = None
    lyrics = None
    player = None

    cover = None
    time = 0
    state = State.CLOSED
    track = EMPTY_TRACK
    _last_track = None

    _menu = None
    _status_icon = None
    _timer_id = None
    _window = None

    def __init__(self):
        super(CoverGloobus, self).__init__()
        self._observers = ([], [], [], [], [])
        self._window = CoverGloobusUI(self)
        self._window.connect('button-press-event', self.clicked)
        self.attach(STATE, self.update_state)
        self.attach(TRACK, self.update_track)

    def main(self):
        try:
            self._lock = open(LOCK_PATH, 'w')
            fcntl.lockf(self._lock, fcntl.LOCK_EX | fcntl.LOCK_NB)
        except:
            c.warning(_('CoverGloobus is already running'), True)
            return 1
        self.init_dbus()

        gobject.idle_add(self.init)

        return gtk.main()

    def init(self):
        self.k_ctrl = False
        #self.cover = None
        self._hide = config.getboolean("General", "hide_if_inactive")

        self.init_lyrics()
        self.init_covers()
        self.init_player()
        self.init_dock()
        self.init_ui()
        self.init_menu()
        self.init_timer()
        self.init_status_icon()

        c.debug('Init done')
        return False

    def init_dbus(self):
        c.debug("Initializing dbus")
        DBUS_NAME = "org.freedesktop.DBus"
        DBUS_PATH = "/org/freedesktop/DBus"

        #SET Global dbus main loop
        dbus.set_default_main_loop(DBusGMainLoop(set_as_default=True))
        session_bus = dbus.SessionBus()
        dbus_object = session_bus.get_object(DBUS_NAME, DBUS_PATH)
        self.dbus_iface = dbus.Interface(dbus_object, DBUS_NAME)
        self.dbus_wrapper = DBusWrapper(self)

    def init_player(self):
        name = config.get("General", "player")
        if self.player == None or name != self.player.get_player_name():
            if self.player != None:
                c.debug("Removing old player")
                self.reset_track()
                self.player.close()
            c.debug("Loading player")
            #self.player = __import__('players.' + name, fromlist=['players']).__dict__[name](self)
            self.player = get_player(name, self)
            c.debug("Player loaded: " + name)
            self.update(PLAYER, self.player)
            self.check_song()

    def init_dock(self):
        if config.getboolean("Docky", "show"):
            if not self.dock:
                self.dock = Dock(self)
            if self.dock:
                self.dock.init()
        else:
            if self.dock:
                self.dock.disconnect()
                self.dock = None

    def init_lyrics(self):
        if not self.lyrics:
            self.lyrics = LyricSearch(self)
        self.lyrics.init()

    def init_covers(self):
        if not self.covers:
            self.covers = CoverSearch(self)
        self.covers.init()

    def init_ui(self):
        self._window.init()
        if self.state == State.CLOSED and self._hide:
            self._window.hide()
        else:
            self._window.show()

    def init_timer(self):
        if self.state < State.CLOSED and 'time' in self.player.capabilities and len(self._observers[TIME]) > 0:
            if not self._timer_id:
                self._timer_id = gobject.timeout_add(500, self.update_time)
                self.update_time()
        elif self._timer_id:
            gobject.source_remove(self._timer_id)
            self._timer_id = None
            if self.time:
                self.time = 0
                self.update(TIME, self.time)

    def init_status_icon(self):
        show = config.getboolean("General","show_status_icon")
        c.debug('Showing status icon: ' + str(show))
        if self._status_icon == None and show:
            #self._status_icon = gtk.status_icon_new_from_file(common.ICON_PATH)
            self._status_icon = gtk.status_icon_new_from_icon_name('covergloobus')
            self._status_icon.set_tooltip("CoverGloobus")
            self._status_icon.connect('popup-menu', self.status_icon_menu)
            if self.state < State.STOPPED:
                self._status_icon.set_tooltip(self.track.title + \
                                              '\nby ' + self.track.artist + \
                                              '\nfrom ' + self.track.album)
        elif self._status_icon != None:
            self._status_icon.set_visible(show)

    def update_config(self):
        x,y = self._window.get_position()
        config.set('Window', 'x', str(x))
        config.set('Window', 'y', str(y))
        config.set('Window', 'ontop', '1' if self._window.ontop else '0')
        config.set('Window', 'stick', '1' if self._window.stick else '0')

    def clicked(self, widget, event):
        c.debug("Button Pressed: " + str(event.button))
        if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
            if self.track.cover:
                __import__('widgets.window', fromlist=['widgets']).__dict__['CoverWindow'](self.track)
        elif event.button == 2:
            self.lyrics.load('lyrics', self.track)
        elif event.button == 3:
            c.debug("Right Button Pressed, Launch menu")
            self._menu.popup(None,None,None,3,event.get_time(),0)

    def status_icon_menu(self, icon, button, time):
        if button == 3:
            self._menu.popup(None, None, gtk.status_icon_position_menu, 3, time,
                             self._status_icon)

    def init_menu(self):
        if self._menu is None:
            self._menu   = gtk.Menu()
            lyrics  = gtk.MenuItem(_("Show Lyrics"))
            self._menu.append (lyrics)
            tabs    = gtk.MenuItem(_("Show Tablatures"))
            self._menu.append (tabs)

            self._menu.append(gtk.SeparatorMenuItem())

            stick   = gtk.CheckMenuItem(_("Stick to Desktop"))
            self._menu.append (stick)
            ontop   = gtk.CheckMenuItem(_("Always on Top"))
            self._menu.append (ontop)
            config  = gtk.MenuItem(_("Configure"))
            self._menu.append (config)

            self._menu.append(gtk.SeparatorMenuItem())

            blog    = gtk.MenuItem(_("Blog"))
            self._menu.append (blog)
            donate  = gtk.MenuItem(_("Donate"))
            self._menu.append (donate)
            bug     = gtk.MenuItem(_("Report a Problem"))
            self._menu.append (bug)
            about   = gtk.MenuItem(_("About"))
            self._menu.append (about)

            self._menu.append(gtk.SeparatorMenuItem())

            quit    = gtk.MenuItem(_("Quit"))
            self._menu.append (quit)

            self._menu.__stick = stick
            self._menu.__ontop = ontop

            lyrics.connect('button-press-event', lambda w, e: self.lyrics.load('lyrics', self.track))
            tabs.connect  ('button-press-event', lambda w, e: self.lyrics.load('tabs', self.track))
            config.connect('button-press-event', lambda w, e: __import__('core.configuration', fromlist=['core']).__dict__['Configuration'].run(self))
            ontop.connect ("activate"          , lambda w: self._window.set_ontop(w.active))
            stick.connect ("activate"          , lambda w: self._window.set_stick(w.active))
            blog.connect  ('button-press-event', lambda w, e: os.system("gnome-open http://gloobus.wordpress.com/"))
            donate.connect('button-press-event', lambda w, e: os.system("gnome-open \"https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=guitarboy000%40gmail%2ecom&item_name=Gloobus%2c%20A%20Quicklook%20for%20linux&no_shipping=0&no_note=1&tax=0&currency_code=EUR&lc=US&bn=PP%2dDonationsBF&chars\""))
            bug.connect   ('button-press-event', lambda w, e: os.system("gnome-open https://bugs.launchpad.net/covergloobus"))
            about.connect ('button-press-event', lambda w, e: __import__('widgets.about', fromlist=['widgets']).__dict__['AboutGloobus'].run())
            quit.connect  ('button-press-event', lambda w, e: self.quit())

            self._menu.show_all()

        self._menu.__ontop.set_active(self._window.ontop)
        self._menu.__stick.set_active(self._window.stick)

    def quit(self):
        c.debug("MENU OPTION: quit")
        if self.dock:
            self.dock.disconnect()
        self.update_config()
        config.save()
        gtk.main_quit()

    def check_song(self):
        """ Checks if player is active or not
            if active load current song, controlls if it is different...
        """
        if self.player.connected:
            state = self.player.get_state()
        else:
            state = State.CLOSED

        if state in (State.PLAYING, State.PAUSED):
            track = self.player.get_track()
        else:
            track = Track()

        self.update(TRACK, track)
        self.update(STATE, state)

    def reset_track(self):
        if not self.track is EMPTY_TRACK:
            self._last_track = self.track
        self.update(TRACK, EMPTY_TRACK)
        return False

    def update_cover(self, artist, album, cover):
        if self.track.artist == artist and self.track.album == album:
            c.debug("Updating cover")
            if self.cover != cover:
                self.update(COVER, cover)
                self.cover = cover

    def update_state(self, state):
        c.debug("state update: " + State.toString(state))
        if self.state != state:
            self.state = state
            if state == State.PLAYING:
                if self.track is EMPTY_TRACK and self._last_track:
                    self.update(TRACK, self._last_track)
            elif state == State.STOPPED:
                self.reset_track()
            elif state == State.STARTED:
                self.check_song()
                self.init_timer()
                self._window.show()
            elif state == State.CLOSED:
                self.reset_track()
                self.init_timer()
                if self.dock:
                    self.dock.reset()
                if self._hide:
                    self._window.hide()


    def update_track(self, track):
        if track is EMPTY_TRACK:
            c.debug("track update: empty")
            if self._status_icon:
                self._status_icon.set_tooltip("CoverGloobus")
            self.update(COVER, None)
            self.cover = None
        else:
            c.debug("track update: %s by %s" % (track.title, track.artist) )
            if not track.cover:
                track.cover = self.covers.search(track)
            if track.cover:
                if self.cover != track.cover:
                    self.update(COVER, track.cover)
                    self.cover = track.cover
            else:
                self.update(COVER, None)
                self.cover = None
            if self._status_icon:
                self._status_icon.set_tooltip(track.title + \
                                              '\nby ' + track.artist + \
                                              '\nfrom ' + track.album)
        self.track = track

    def update_time(self):
        time = self.player.get_position()
        if self.time != time:
            self.time = time
            self.update(TIME, time)
        return True

    def attach(self, target, callback, *args):
        ob = Observer(target, callback, *args)
        self._observers[target].append(ob)
        if target == TIME:
            self.init_timer()
        return ob

    def detach(self, observer):
        self._observers[observer.target].remove(observer)
        if observer.target == TIME:
            self.init_timer()

    def update(self, target, *args):
        for callback in self._observers[target]:
            callback(*args)
