# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gobject
import logging
import os
import shutil
import glob

from threading import Thread

from gettext import gettext as _

import common

from core import config
from core.common import CACHE_PATH
from core.utils import name_hash

from covers import GenericCover

c = logging.getLogger(__name__)

COVERS_PATH = CACHE_PATH + "covers/"
if not os.path.exists(COVERS_PATH):
    os.mkdir(COVERS_PATH)

class CoverSearch:
    """ Covers searching and downloading class """
    _local_engines = None
    _cloud_engines = None
    _search_queue = None

    update_cover = None

    def __init__(self, cg):
        self._search_queue = []
        self.cg = cg

    def init(self):
        """ Loads search engines defined in configuration. """
        self._local_engines = []
        self._cloud_engines = []
        for klass in config.get('Covers', 'search_engines').split(','):
            c.debug('Loading engine ' + klass)
            if klass:
                try:
                    engine = __import__('covers.'+klass, fromlist=['covers']).__dict__[klass]()
                    if engine.__local__:
                        self._local_engines.append(engine)
                    else:
                        self._cloud_engines.append(engine)
                except:
                    c.debug("Error while loading class " + klass)

    def search(self, track):
        if not track.artist and not track.album:
            return None

        for engine in self._local_engines:
            cover = engine.get_cover(track)
            if cover:
                return cover

        cover = self._check_cache(track.album, track.artist)
        if cover:
            return cover

        data = (track.artist, track.album)
        if self._search_queue.count(data) > 0:
            c.debug("Already downloading this cover")
        else:
            self._search_queue.append(data)
            loader = Thread(target=self._load_cover, args=(track,))
            loader.start()
        return None

    def _check_cache(self, album, artist):
        filename = name_hash(artist, True) + "-" + name_hash(album)
        for path in glob.glob( os.path.join(COVERS_PATH, filename+'.*') ):
            c.debug("Cover found in cache: " + path)
            return path
        return None

    def _load_cover(self, track):
        for engine in self._cloud_engines:
            cover = engine.get_cover(track)

            if cover:
                dot = cover.rfind('.')
                if dot > -1:
                    ext = cover[dot:]
                else:
                    ext = ''
                path = COVERS_PATH + name_hash(track.artist, True) + "-" + \
                                     name_hash(track.album) + ext
                shutil.move(cover, path)
                gobject.idle_add(self._set_cover, track.artist, track.album, path)
                return path

    def _set_cover(self, artist, album, cover):
        self.cg.update_cover(artist, album, cover)
        self._search_queue.remove((artist, album))
        return False
