# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# todo: window constant size ???
# todo:  change 'Go to webpage' to 'Add lyrics' when lyrics not found
#        for sites that support adding lyrics

import gobject
import logging
import os
import re
import string
import sys
import threading

from threading import Thread
from threading import Lock

from gettext import gettext as _

from core import config

from core.common import CACHE_PATH
from core.common import GLOOBUS_URL
from core.common import TRACK

from core.utils import name_hash

from lyrics import GenericLyric
from tabs import GenericTab
from widgets.window import LyricWindow

c = logging.getLogger(__name__)

path = os.path.join(CACHE_PATH, 'lyrics')
if not os.path.exists(path):
    os.mkdir(path)

path = os.path.join(CACHE_PATH, 'tabs')
if not os.path.exists(path):
    os.mkdir(path)

class LyricSearch:
    """ Lyrics/tabs searching and dispalying class """
    def __init__(self, cg):
        self.last_loaded = None
        self._lyric_engines = []
        self._tab_engines = []
        self.lyric_engines = {}
        self.tab_engines = {}
        self.windows = {
            'lyrics'  : None,
            'tabs'   : None,
            'shared' : None
        }

        cg.attach(TRACK, self.set_track)

        c.debug('Init done')

    def init(self):
        """Parses configuration and applys required changes to windows"""
        self.conf_window = config.get("Lyrics","window")
        if self.conf_window == 'new':
            for n, w in self.windows.items():
                if w: w.destroy()
        elif self.conf_window == 'separate':
            if self.windows['shared']:
                self.windows['shared'].destroy()
        elif self.conf_window == 'shared':
            if self.windows['lyrics']:
                self.windows['lyrics'].destroy()
            if self.windows['tabs']:
                self.windows['tabs'].destroy()
        self.widget = config.getboolean('Lyrics', 'widget')
        for n, w in self.windows.items():
            if w: w.set_widget(self.widget)
        self.refresh = config.getboolean('Lyrics', 'refresh')
        self.cache = config.getboolean('Lyrics', 'cache')

        # loading lyrics engines
        for klass in config.get('Lyrics', 'lyrics_source').split(','):
            if klass:
                try:
                    engine = __import__('lyrics.'+klass, fromlist=['lyrics']).__dict__[klass]()
                    self._lyric_engines.append(engine)
                except:
                    c.debug("Error loading class " + klass)
        # loading tabs engines
        for klass in config.get('Lyrics', 'tabs_source').split(','):
            if klass:
                try:
                    engine = __import__('tabs.'+klass, fromlist=['tabs']).__dict__[klass]()
                    self._tab_engines.append(engine)
                except:
                    c.debug("Error loading class " + klass)

    def set_track(self, track):
        """ If windows are open load new lyrics/tabs """
        if self.refresh:
            if self.windows['lyrics']:
                self.load('lyrics', track)
            if self.windows['tabs']:
                self.load('tabs', track)
            if self.windows['shared']:
                self.load(self.last_loaded, track)

    def load(self, target, track):
        """ Loads lyrics/tabs, creates a new window if needed,
            creates a new thread for every lyrics/tabs.
        """
        self.last_loaded = target
        c.debug('Loading ' + target)
        if self.conf_window == 'new':
            window = LyricWindow()
            window.connect('destroy', self.remove_window, None)
        elif self.conf_window == 'separate':
            if not self.windows[target]:
                self.windows[target] = LyricWindow()
            window = self.windows[target]
            window.connect('destroy', self.remove_window, target)
        elif self.conf_window == 'shared':
            if not self.windows['shared']:
                self.windows['shared'] = LyricWindow()
            window = self.windows['shared']
            window.connect('destroy', self.remove_window, 'shared')

        window.set_widget(self.widget)
        if window.loader:
            self.remove_window(window,None)

        if target == 'lyrics':
            not_found = _('No lyrics found')
            window.set_title(_('CoverGloobus Lyrics'))
            engines = self._lyric_engines
        elif target == 'tabs':
            not_found = _('No tablatures found')
            window.set_title(_('CoverGloobus Tabs'))
            engines = self._tab_engines

        if track.title != '' and track.artist != '':
            window.set_text(_('Searching ...'), GLOOBUS_URL)
            loader = LyricLoader(track, window, engines, not_found, self.cache, target)
            window.set_loader(loader)
            loader.start()
        else:
            window.set_text(_('No song playing'), GLOOBUS_URL)
        window.show_all()

    def remove_window(self, widget, target):
        """ When window is closed stops its thread and assigns its pointer to None
        """
        if widget.loader:
            widget.loader.stop()
        if target:
            self.windows[target] = None

# ============================== LYRIC LOADER ================================ #

class LyricLoader(Thread):
    """ Threaded lyrics/tabs loader.
        After downloading, places the result in its window.
    """
    def __init__(self, track, window, engines, not_found, cache, target):
        super(LyricLoader, self).__init__()
#        .Thread.__init__(self)
        self.track = track
        self.window = window
        self.engines = engines
        self.not_found = not_found
        self.cache = cache
        self.target = target

        self.load = True
        self.load_lock = Lock()
        self.cached = False

    def run(self):
        """Main thread function"""
        data = self.check_cache()
        if data:
            source = data[0]
            url = data[1]
            text = data[2]
        else:
            source = ''
            url = GLOOBUS_URL
            text = None

        if not text:
            for engine in self.engines:
                source = engine.__class__.__name__
                c.debug('Trying ' + source)
                try:
                    text = engine.get(self.track.artist, self.track.title)
                    if text:
                        c.debug('Found in ' + source)
                        url = engine.get_webpage()
                        break
                except Exception as e:
                    c.error('Error while downloading %s - %s, %s' % (self.target, str(type(e)), str(e)))
        else:
            c.debug(self.target + ' found in cache')

        if not text:
            c.debug('Lyric not found')
            text = self.not_found
        elif not self.cached:
            self.save_cache(text, url, source)

        self.check_load()
        gobject.idle_add(self.window.set_text, text, url)

    def check_load(self):
        """ Checks if thread should continue its work """
        with self.load_lock:
            if not self.load:
                raise SystemExit

    def check_cache(self):
        """ Checks if song lyrics/tabs are in cache """
        if self.cache:
            c.debug('Checking cache')
            PATH = CACHE_PATH + self.target + '/'
            filename = name_hash(self.track.artist, True) + ' - ' + name_hash(self.track.title)
            path = os.path.join(CACHE_PATH, self.target, filename)
            if os.path.exists(path):
                with open(path, 'r') as f:
                    source = f.readline()
                    url = f.readline()
                    data = f.read()
                    self.cached = True
                    return (source, url, data)
        return None

    def save_cache(self, data, url, source):
        """Saves songs lyrics/tabs in cache"""
        if self.cache:
            c.debug('saving cache')
            filename = name_hash(self.track.artist, True) + ' - ' + name_hash(self.track.title)
            path = os.path.join(CACHE_PATH, self.target, filename)
            with open(path, 'w+') as f:
                data = source + '\n' + url + '\n' + data
                f.write(data)
                self.cached = True

    def stop(self):
        """ Stops the thread """
        with self.load_lock:
            if self.load:
                self.load = False
