# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import logging
import os
import re

from string import capwords
from urllib2 import quote
from urllib2 import urlopen
from xml.dom import minidom
from gettext import gettext as _

from core.utils import html_unescape
from core.utils import name_hash
from lyrics import GenericLyric

c = logging.getLogger(__name__)

WIKIA = 'http://lyrics.wikia.com/'
CACHE_PATH = '/tmp/cg_lyricswikia_cache/'
QUEUE_MAX = 10

URL_TEMPLATE = "http://lyrics.wikia.com/%s:%s"
ARTIST_API = 'http://lyrics.wikia.com/api.php?func=getArtist&artist=%s&fmt=xml&fixXML'

class lyricswikia2(GenericLyric):

    __title__ = 'LyricsWikia 2'
    __version__ = '0.2.3'
    __authors__ = 'BadChoice <guitarboy000@gmail.com>, Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('Download lyrics from Lyrics.Wikia.com')

    url = ''

    _artists = {}
    _queue = []
    _cache_mtime = 0

    def __init__(self):
        if not os.path.exists(CACHE_PATH):
            os.mkdir(CACHE_PATH)

    def get(self, artist, title):
        self.url = self._get_song_link(artist, title)
        if not self.url:
            c.debug('url not found, using the old method')
            artist = capwords(artist.replace('.','.  ')).replace('.  ','.')
            artist = quote(artist.replace(" ","_").encode('utf-8'))
            title = capwords(title.replace('.','.  ')).replace('.  ','.')
            title = quote(title.replace(" ","_").encode('utf-8'))
            self.url = URL_TEMPLATE % (artist, title)

        c.debug('lyrics.wikia URL: ' + self.url)
        web = urlopen(self.url)
        webcode = web.read()

        #<div class='lyricbox'>  ==> Lyric start
        #(.*)                    ==> Lyric itself with some divs
        #(?:<!--(.*?)<p>)        ==> End of the lyric
        regexp = re.compile("<div\s+class='lyricbox'>(.*)(?:<!--.*?<p>)",
                            re.DOTALL)
        m = regexp.search(webcode)

        if m:
            lyric = m.group(1).replace("<br />", "\n")

            #(?:<div.*>)         ==> Leading divs
            #(.*)                ==> Finally the lyric
            regexp = re.compile ("(?:<div.*>)(.*)",re.DOTALL)
            m = re.search(regexp, lyric)
            if m:
                lyric =  m.group(1)

            lyric = html_unescape(lyric)
            lyric = re.sub('</?.+?>', '', lyric)
            return lyric
        else:
            return None

    def get_webpage(self):
        return self.url

    def _get_song_link(self, artist, title):
        artist_hash = name_hash(artist, True)
        if artist_hash in self._artists:
            artist_data = self._artists[artist_hash]
        else:
            if os.path.exists(CACHE_PATH + artist_hash):
                artist_data = self._load_cache(artist_hash)
            else:
                artist = capwords(artist.replace('.','.  ')).replace('.  ','.')
                artist = quote(artist.replace(" ","_").encode('utf-8'))
                xml = urlopen(ARTIST_API % artist).read()
                artist_data = self._parse_xml(xml)
            self._artists[artist_hash] = artist_data
            if artist_data:
                self._queue.append(artist_hash)
                if len(self._queue) > QUEUE_MAX:
                    self._update_cache()
        if artist_data:
            title_hash = name_hash(title)
            if title_hash in artist_data.songs:
                artist = quote(artist_data.link.encode('utf-8'))
                title = quote(artist_data.songs[title_hash].encode('utf-8'))
                return URL_TEMPLATE % (artist, title)
        return None

    def _load_cache(self, artist_hash):
        with open(CACHE_PATH + artist_hash, 'r') as f:
            data = _ArtistData
            data.link = f.readline()[:-1]
            data.songs = {}
            for line in f:
                title_hash, song_link = line[:-1].split('|')
                data.songs[title_hash] = song_link
            return data
        return None

    def _update_cache(self):
        artist_hash = self._queue.pop(0)
        artist_data = self._artists.pop(artist_hash)
        with open(CACHE_PATH + artist_hash, 'w+') as f:
            f.write(artist_data.link + '\n')
            for title_hash, title in artist_data.songs.iteritems():
                f.write(title_hash + '|' + title + '\n')

    def _parse_text(self, nodelist):
        rc = []
        for node in nodelist:
            if node.nodeType == node.TEXT_NODE:
                rc.append(node.data)
        return ''.join(rc)

    def _parse_xml(self, xml):
        data = _ArtistData
        data.songs = {}
        dom = minidom.parseString(xml)
        artist = dom.getElementsByTagName('artist')[0]
        albums = dom.getElementsByTagName('albums')[0]
        for album in albums.getElementsByTagName('albumResult'):
            songs = album.getElementsByTagName('songs')[0]
            for song in songs.getElementsByTagName('item'):
                title = self._parse_text(song.childNodes)
                data.songs[name_hash(title)] = title
        if len(data.songs) > 0:
            data.link = self._parse_text(artist.childNodes)
            return data
        return None

class _ArtistData(object):
    link = None
    songs = None
