# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import logging
import os

from urllib import unquote

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import COVER, STATE, TIME, TRACK
from generic.Player import Player

c = logging.getLogger(__name__)

class Banshee(Player):

    __title__ = 'Banshee'
    __version__ = '0.1'
    __authors__ = 'BadChoice <guitarboy000@gmail.com>, Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('Banshee music player plugin')

    DBUS_NAME   = 'org.bansheeproject.Banshee'
    DBUS_PLAYER = '/org/bansheeproject/Banshee/PlayerEngine'
    DBUS_CONTROLLER = '/org/bansheeproject/Banshee/PlaybackController'

    capabilities = ('rating', 'time', 'seek', 'volume')

    _controller = None
    _player = None
    _signals = []
    _state = None

    def connect(self):
        """
            Connects player
            @return: boolean, True if successful, False if not
        """
        try:
            if not self.connected:
                proxy = dbus.SessionBus().get_object(self.DBUS_NAME,
                    self.DBUS_PLAYER)
                self._player = dbus.Interface(proxy,
                    'org.bansheeproject.Banshee.PlayerEngine')

                proxy = dbus.SessionBus().get_object(self.DBUS_NAME,
                    self.DBUS_CONTROLLER)
                self._controller = dbus.Interface(proxy,
                    'org.bansheeproject.Banshee.PlaybackController')

                self._signals.append(self._player.connect_to_signal(
                    'StateChanged', self.state_changed))
                self._signals.append(self._player.connect_to_signal(
                    'EventChanged', self.event_changed))
                self.connected = True
        except:
            self.disconnect()
        return self.connected

    def disconnect(self):
        """
            Disconnects player
        """
        for signal in self._signals:
            signal.remove()

        self._player = None
        self._controller = None
        self._signals = []
        self.connected = False

    def get_cover_path(self, art_id=None):
        """
            @param filename: string, contains path to current playing song.
                optional
            @return: string, path to the album art image of current playing
                song, (not URI)
        """
        try:
            if not art_id:
                art_id = self._player.GetCurrentTrack()['artwork-id']
            path = "~/.cache/media-art/" + art_id +".jpg"
            path = os.path.expanduser(path)
            if not os.path.exists(path):
                path = "~/.cache/album-art/" + art_id +".jpg"
            path = os.path.expanduser(path)
            if os.path.exists(path):
                return path
        except:
            pass
        return ''

    def get_state(self, state=None):
        if not state:
            try:
                state = self._player.GetCurrentState()
            except:
                return State.CLOSED
        if state == 'idle' or state == 'stopped':
            return State.STOPPED
        elif state == 'playing':
            return State.PLAYING
        elif state == 'paused':
            return State.PAUSED

    def previous(self):
        """
            Changes song to previous
        """
        if self.connected:
            self._controller.Previous(True)

    def play(self):
        """
            Plays or pauses the player
        """
        if self.connected:
            self._player.TogglePlaying()

    def next(self):
        """
            Changes song to next
        """
        if self.connected:
            self._controller.Next(True)

    def get_track(self, data=None):
        """
            Prepares a Track object with current playing song data

            @param data: dictionary, contains info about current playing song.
                optional
            @return: Track, object with current playing song data
        """
        if not data:
            data = self._player.GetCurrentTrack()

        vals = ('name','album','artist', 'genre', 'year','track-number',
                'URI', 'rating', 'artwork-id', 'length')
        for key in [key for key in vals if key not in data]:
            c.warning('%s: %s not found in current track' % ('BANSHEE', key))
            data[key] = ''

        track = Track(
            title  = data['name'],
            album  = data['album'],
            artist = data['artist'],
            genre  = data['genre'],
            year   = data['year'],
            rating = data['rating'],
            tracknumber = data['track-number'],
            cover  = self.get_cover_path(data['artwork-id']),
            location = unquote(data['URI'].encode('UTF-8')),
            length = data['length']
        )

        return track

    def get_position(self):
        try:
            return self._player.GetPosition() / 1000
        except:
            return 0

    def set_position(self, time):
        self._player.SetPosition(dbus.UInt32(time*1000))

    def get_volume(self):
        try:
            return self._player.GetVolume()
        except:
            return 0

    def set_volume(self, volume):
        self._player.SetVolume(dbus.UInt16(volume))

    def set_rating(self, rating):
        if 0 <= rating and rating <= 5:
            self._player.SetRating(dbus.Byte(rating))

# ================================= HANDLERS ================================= #

    def state_changed(self, message, *args):
        state = self.get_state(message)
        if state in (State.PLAYING, State.PAUSED):
            self._state = state
        elif self._state == State.PLAYING and state == State.STOPPED:
            return
        self.cg.update(STATE, state)

    def event_changed(self, message, *args):
        if message in ('startofstream', 'trackinfoupdated'):
            self.cg.update(TRACK, self.get_track())
