# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import logging
import urllib

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import TRACK, STATE
from generic.Player import Player

c = logging.getLogger(__name__)

class Exaile3(Player):

    __title__ = 'Exaile 0.3.x'
    __version__ = '0.1'
    __authors__ = 'Rune Svendsen <runesvend@gmail.com>, Jaap Broekhuizen <jaapz.b@gmail.com>, Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('Exaile 0.3.x music player plugin. Requies Exaile 0.3.2 or newer')

    DBUS_NAME   = 'org.exaile.Exaile'
    DBUS_OBJECT = '/org/exaile/Exaile'

    _player = None
    _signals = []

    capabilities = ('rating', 'time', 'seek', 'volume')

    def connect(self):
        try:
            if not self.connected:
                proxy = dbus.SessionBus().get_object(self.DBUS_NAME,
                    self.DBUS_OBJECT)
                self._player = dbus.Interface(proxy, self.DBUS_NAME)

                self._signals.append(self._player.connect_to_signal(
                    'StateChanged', self.state_changed))
                self._signals.append(self._player.connect_to_signal(
                    'TrackChanged', self.track_changed))
                self.connected = True
        except:
            self.disconnect()

        return self.connected

    def disconnect(self):
        for signal in self._signals:
            signal.remove()

        self._player = None
        self._signals = []
        self.connected = False

    def get_rating(self):
        """
            @return: integer, [0-5] value, representing rating of current
                playing song
        """
        # GetTrackAttr('__rating')) returns a string describing the rating in percent.
        # This string includes a decimal point and a zero following the decimal point,
        # so 3 out of 5 starts returns "60.0"
        # this code works assuming that the rating that get_rating should
        # return is "number of stars out of 5"
        try:
            return int((float(self._player.GetTrackAttr('__rating')) * 5) / 100)
        except:
            return 0

    def get_state(self):
        return State.get(self._player.GetState())

    def get_track(self):
        data = {
                'title' : self._player.GetTrackAttr('title'),
                'album' : self._player.GetTrackAttr('album'),
                'artist': self._player.GetTrackAttr('artist'),
                'rating': self.get_rating(),
                'genre' : self._player.GetTrackAttr('genre'),
                'tracknumber' : self._player.GetTrackAttr('tracknumber'),
                'year' : self._player.GetTrackAttr('date'),
                'location' : urllib.unquote(self._player.GetTrackAttr('__loc').encode('utf-8')),
                'rating' : self.get_rating(),
                'length' : int(float(self._player.GetTrackAttr('__length')))
            }
        return Track(**data)

    def set_rating(self, rating):
        if 0 <= rating and rating <= 5:
            rating *= 20.0
            self._player.SetTrackAttr('__rating', rating)

    def get_position(self):
        data = str(self._player.CurrentPosition()).split(':')
        time = 0
        for t in data:
            time = time * 60 + int(t)
        return time

    def set_position(self, time):
        self._player.Seek(float(time))

    def get_volume(self):
        return int(float(self._player.GetVolume()))

    def set_volume(self, volume):
        self._player.ChangeVolume(volume)

    def previous(self):
        """
            Changes song to previous
        """
        if self.connected:
            self._player.Prev()

    def play(self):
        """
            Plays or pauses the player
        """
        if self.connected:
            self._player.PlayPause()

    def next(self):
        """
            Changes song to next
        """
        if self.connected:
            self._player.Next()

# ================================= HANDLERS ================================= #

    def state_changed(self):
        self.cg.update(STATE, self.get_state())

    def track_changed(self):
        self.cg.update(TRACK, self.get_track())
