# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gobject
import gtk
import logging

from gettext import gettext as _

from core import config
from core.track import State
from core.track import Track
from core.common import TRACK, STATE
from generic.Player import Player

c = logging.getLogger(__name__)

try:
    import mpd
except ImportError:
    import sys
    c.error(_("python-mpd is non installed.\nInstall it or run covergloobus-config to select another player."), True)
    sys.exit(1)

class MPD(Player):
    __title__ = 'MPD'
    __version__ = '0.0'
    __authors__ = 'Jaap Broekhuizen <jaapz.b@gmail.com>'
    __desc__ = _('Music Player Daemon plugin')

    _client = None
    _data = None
    _state = ''
    _track = Track()
    _timeout_id = None

    def is_active(self):
        try:
            if self._client:
                self._client.status()
                return True
        except:
            pass
        return self._connect()

    def _connect(self):
        host = config.get('MPD', 'host')
        port = config.getint('MPD', 'port')
        password = config.get('MPD', 'password')
        self._path = config.get('MPD', 'path')
        if self._path[-1:] == '/':
            self._path = self._path[:-1]
        try:
            if self._client:
                c.debug('was connected, disconnecting')
                self._client.disconnect()
            self._client = mpd.MPDClient()
            c.debug('connecting')
            self._client.connect(host, port)
            if password != '':
                c.debug('using password')
                self._client.password(password)
            c.debug('connection done')
            return True
        except:
            c.debug('connection failed')
            return False

    def connect(self):
        if not self._timeout_id:
            self._timeout_id = gobject.timeout_add(2000, self.check_new_song)
        self.check_new_song()
        self.connected = True
        return True

    def disconnect(self):
        if self._client:
            try:
                self.client.disconnect()
            except:
                pass
        self.connected = False
        if self._timeout_id:
            gobject.source_remove(self._timeout_id)
            self._timeout_id = 0

    def check_new_song(self):
        try:
            status = self._client.status()
        except:
            if self.is_active():
                status = self._client.status()
            else:
                self._state = ''
                self.cg.update(STATE, State.CLOSED)
                return True

        if self._state != status['state']:
            self._state = status['state']
            self.cg.update(STATE, self.get_state())

        try:
            data = self._client.currentsong()
            if self._data != data:
                self._data = data
                self.cg.update(TRACK, self._get_track())
        except:
            pass
        return True

    def _get_track(self):
        data = {}
        vals = ('title','album','artist', 'genre', 'date', 'track', 'file')
        for key in vals:
            if key in self._data:
                data[key] = self._data[key]
            else:
                data[key] = ''

        data['file'] = '%s/%s' % (self._path, data['file'])
        track = Track(
            title  = data['title'],
            album  = data['album'],
            artist = data['artist'],
            genre  = data['genre'],
            year   = data['date'],
            tracknumber = data['track'],
            location = data['file']
        )
        return track

    def get_track(self):
        return self._get_track()

    def get_state(self):
        """
            Returns players actual state.
            @return: string - playing, paused, stopped
        """
        if self._state == 'play':
            return State.PLAYING
        if self._state == 'pause':
            return State.PAUSED
        if self._state == 'stop':
            return State.STOPPED
        else:
            return State.CLOSED

    def previous(self):
        if self.connected:
            self._client.previous()

    def next(self):
        if self.connected:
            self._client.next()

    def play(self):
        if self.connected:
            status = self._client.status()
            if status['state'] == 'play':
                self._client.pause(1)
            else:
                self._client.pause(0)

    @staticmethod
    def configure(dialog):
        mpd_box = dialog.get_content_area()
        mpd_box.set_border_width(10)
        LABEL_WIDTH = 120
        ENTRY_WIDTH = 200

        box = gtk.HBox()
        label = gtk.Label(_("Host:"))
        label.set_alignment(0,0.5)
        label.set_size_request(LABEL_WIDTH, -1)
        host = gtk.Entry()
        host.set_text(config.get('MPD','host'))
        host.set_size_request(ENTRY_WIDTH, -1)
        box.pack_start(label, False)
        box.pack_start(host, False)
        mpd_box.pack_start(box,False)

        box = gtk.HBox()
        label = gtk.Label(_("Port:"))
        label.set_alignment(0,0.5)
        label.set_size_request(LABEL_WIDTH, -1)
        port = gtk.Entry()
        port.set_text(config.get('MPD','port'))
        port.set_size_request(ENTRY_WIDTH, -1)
        box.pack_start(label,False)
        box.pack_start(port, False)
        mpd_box.pack_start(box,False)

        box = gtk.HBox()
        label = gtk.Label(_("Password:"))
        label.set_alignment(0,0.5)
        label.set_size_request(LABEL_WIDTH, -1)
        password = gtk.Entry()
        password.set_text(config.get('MPD','password'))
        password.set_size_request(ENTRY_WIDTH, -1)
        box.pack_start(label,False)
        box.pack_start(password, False)
        mpd_box.pack_start(box,False)

        box = gtk.HBox()
        label = gtk.Label(_("Library:"))
        label.set_alignment(0,0.5)
        label.set_size_request(LABEL_WIDTH, -1)
        path = gtk.Entry()
        path.set_text(config.get('MPD','path'))
        path.set_size_request(ENTRY_WIDTH, -1)
        box.pack_start(label,False)
        box.pack_start(path, False)
        mpd_box.pack_start(box,False)
        mpd_box.show_all()

        if dialog.run() == gtk.RESPONSE_ACCEPT:
            config.set('MPD', 'host', host.get_text())
            config.set('MPD', 'port', port.get_text())
            config.set('MPD', 'password', password.get_text())
            config.set('MPD', 'path', path.get_text())
