# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import logging
import os
import subprocess

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import TRACK, STATE
from generic.Player import Player

c = logging.getLogger(__name__)

class QuodLibet(Player):
    """
        QuodLibet player plugin
    """
    __title__ = 'QuodLibet'
    __version__ = '0.3'
    __authors__ = 'Daniel Ribeiro <danielrmt@gmail.com>, Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('QuodLibet music player plugin')

    DBUS_NAME   = 'net.sacredchao.QuodLibet'
    DBUS_OBJECT = '/net/sacredchao/QuodLibet'

    capabilities = ('rating', 'time', 'volume', 'seek')

    _player = None
    _signals = []

    def connect(self):
        """
            Connects player
            @return: boolean, True if successful, False if not
        """
        try:
            if not self.connected:
                proxy = dbus.SessionBus().get_object(self.DBUS_NAME, self.DBUS_OBJECT)
                self._player = dbus.Interface(proxy, self.DBUS_NAME)

                self._signals.append(self._player.connect_to_signal(
                    'Unpaused', self.unpaused))
                self._signals.append(self._player.connect_to_signal(
                    'Paused', self.paused))
                self._signals.append(self._player.connect_to_signal(
                    'SongStarted', self.song_started))
                self.connected = True
        except:
            self.disconnect()
        return self.connected

    def disconnect(self):
        for signal in self._signals:
            signal.remove()

        self._player = None
        self._signals = []
        self.connected = False

    def get_rating(self):
        """
            @return: integer, [0-5] value, representing rating of current
                playing song
        """
        try:
            rating = float(self._player.CurrentSong()['~#rating'])
            return round( 5.0 * rating )
        except:
            return 0

    def get_state(self):
        """
            Returns players actual state.
            @return: string - playing, paused, stopped
        """
        if self._player.IsPlaying():
            return State.PLAYING
        else:
            try:
                self._player.CurrentSong()
                return State.PAUSED
            except:
                return State.STOPPED

    def get_track(self, data=None):
        """
            Prepares a Track object with current playing song data

            @param data: dictionary, contains info about current playing song.
                optional
            @return: Track, object with current playing song data
        """
        if not data:
            data = self._player.CurrentSong()

        vals = ('title','album','artist', 'genre', 'date', 'tracknumber',
                '~#rating', '~#length')
        for key in [key for key in vals if key not in data]:
            c.warning('%s: %s not found in current track' % ('QUODLIBET', key))
            data[key] = ''

        track = Track(
            title = data['title'],
            album = data['album'],
            artist= data['artist'],
            rating= round(5*float(data['~#rating'])) if data['~#rating'] else 0,
            genre = data['genre'],
            year  = data['date'],
            tracknumber = data['tracknumber'],
            location = self.getProperty('~filename'),
            length = data['~#length']
        )
        return track

    def previous(self):
        """
            Changes song to previous
        """
        self._player.Previous()

    def play(self):
        """
            Plays or pauses the player
        """
        self._player.PlayPause()


    def next(self):
        """
            Changes song to next
        """
        self._player.Next()

    def set_rating(self, rating):
        try:
            if 0 <= rating and rating <= 5:
                rating = str( rating / 5.0 )
                if self.connected:
                    os.system('quodlibet --set-rating ' + rating)
        except:
            pass

    def get_position(self):
        return int(self._player.GetPosition() / 1000)

    def set_position(self, position):
        os.system('quodlibet --seek=' + str(position))

    def get_volume(self):
        try:
            return int(float(self.systemOutput('quodlibet --status').split(' ')[2]) * 100)
        except:
            return 0

    def set_volume(self, volume):
        os.system('quodlibet --volume=' + str(volume))

# ============================== PRIVATE METHODS ============================= #

    def getProperty(self, name):
        """
            Get playing music property
        """
        try:
            return self.systemOutput('quodlibet --print-playing "<%s>"' % name)
        except:
            return ''

    def systemOutput(self, command):
        """
            Get output from subprocess
        """
        process = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
        os.waitpid(process.pid, 0)
        return process.stdout.read().strip()

# ================================= HANDLERS ================================= #

    def unpaused(self):
        """
            Handles Unpaused dbus signal, calls state_handler()
        """
        self.cg.update(STATE, State.PLAYING)

    def paused(self):
        """
            Handles Paused dbus signal, calls state_handler()
        """
        self.cg.update(STATE, State.PAUSED)

    def song_started(self, message):
        """
            Handles TrackChanged dbus signal, calls state_handler()

            @param message: dictionary with current playing song stats
        """
        self.cg.update(TRACK, self.get_track(message))
