# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gobject
import gtk
import logging
import os

from subprocess import Popen, PIPE
from urllib import unquote
from gettext import gettext as _

from core import config
from core.track import State
from core.track import Track
from core.common import TRACK, STATE, TIME
from generic.Player import Player

c = logging.getLogger(__name__)

try:
    import xmmsclient
    import xmmsclient.glib
except ImportError:
    import sys
    c.error(_("python-xmmsclient is non installed.\n") +
        _("Install it or run covergloobus-config to select another player."),
        True)
    sys.exit(1)

class Xmms2client(Player):
    __title__ = 'Xmms2'
    __version__ = '0.1'
    __authors__ = 'calendros <calendros-lp@laposte.net>'
    __desc__ = _('Xmms2 player plugin')

    capabilities = ('rating', 'time', 'seek', 'volume')

    DBUS_NAME = None
    DBUS_OBJECT = None

    TIMEOUT = 2000

    __async = xmmsclient.XMMS("covergloobus")
    __async_connector = None
    __id = None

    _state = ''
    _track = Track()
    _position = 0
    _volume = 0
    _pid = None
    _timeout_id = None

    def __init__(self, cg):
        super(Xmms2client, self).__init__(cg)
        _timeout_id = gobject.timeout_add(self.TIMEOUT, self.__timeout_cb)

    def is_active(self):
        if Popen(['pidof', 'xmms2d'], stdout=PIPE).communicate()[0].strip():
            return True
        return False

    def connect(self):
        """
        Attempt to connect to the XMMS2 daemon.
        """

        if self.__async_connector:
            c.debug('xmmsclient was already connected')
            return True

        try:
            c.debug("Attempting to connect to XMMS2")

            path = os.getenv("XMMS_PATH")
            self.__async.connect(path, self.__disconnect_cb)
            self.__async_connector = xmmsclient.glib.GLibConnector(
                    self.__async)

            c.debug('Connection established')

            self.__async.broadcast_playback_status(self.__playback_status_cb)
            self.__async.broadcast_playlist_current_pos(
                    self. __playlist_current_pos_cb)
            self.__async.broadcast_playback_current_id(
                    self.__playback_current_id_cb)
            self.__async.broadcast_medialib_entry_changed(
                    self.__medialib_entry_changed_cb)
            self.__async.broadcast_playback_volume_changed(
                    self.__volume_changed_cb)
            self.__async.signal_playback_playtime(self.__playback_playtime_cb)
            self.__async.playback_status(self.__playback_status_cb)
            self.__async.playlist_current_pos(None,
                    self.__playlist_current_pos_cb)
            self.__async.playback_current_id(self.__playback_current_id_cb)

            if not self._timeout_id:
                self._timeout_id = gobject.timeout_add(self.TIMEOUT,
                                                       self.__timeout_cb)

            self.connected = True

            return True
        except IOError:
            c.debug('Connection failed')
            return False

    def disconnect(self):
        c.debug("Disconnecting from XMMS2")
        if self._timeout_id:
            gobject.source_remove(self._timeout_id)
            self._timeout_id = None
        if self.__async:
            try:
                self.__async.disconnect()
            except:
                pass
        self.__async_connector = None
        self.__id = None
        self.connected = False

    def get_state(self):
        """
            Returns players actual state.
            @return: string - playing, paused, stopped
        """

        if self._state == 'play':
            return State.PLAYING
        if self._state == 'pause':
            return State.PAUSED
        if self._state == 'stop':
            return State.STOPPED
        else:
            return State.CLOSED

    def get_track(self):
        return self._track

    def previous(self):
        if self.__async_connector:
            self.__async.playlist_set_next_rel(-1,
                    lambda result: self.__async.playback_tickle())
    def play(self):
        if self.__async_connector:
            if self._state == 'play':
                self.__async.playback_pause()
            else:
                self.__async.playback_start()

    def next(self):
        if self.__async_connector:
            self.__async.playlist_set_next_rel(1,
                    lambda result: self.__async.playback_tickle())

    def mute(self):
        set_volume(0)

    def set_rating(self, rating):
        if self.__async_connector:
            if self.__id and 0 <= rating and rating <= 5:
                self.__async.medialib_property_set(
                        self.__id, 'rating', rating, 'client/generic')

    def get_position(self):
        return self._position

    def set_position(self, position):
        if self.__async_connector:
            self.__async.playback_seek_ms(position * 1000)

    def get_volume(self):
        return _volume

    def set_volume(self, volume):
        if self.__async_connector:
            volume = min(volume, 100)
            volume = max(volume, 0)

            # TODO other way to set "all" channels or global volume ?
            for chan in ("right", "left"):
                self.__async.playback_volume_set(chan, volume)


# ------------------------------ CALLBACK PART ------------------------------ #

    def __disconnect_cb(self, unknown):
        """
        Clean up after the XMMS2 daemon quits.
        """

        c.debug("Connection lost")

        # TODO: Need to clean up after the connector?
        self.__async_connector = None
        self.__id = None
        self.connected = False
        state = ''
        self.cg.update(STATE, self.get_state())


    def __timeout_cb(self):
        if not self.__async_connector:
            if self.is_active(): self.connect()
        return True # launch the timeout callback again

    def __playback_status_cb(self, result):
        """
        Update the cached status to reflect that of XMMS2.
        """

        mapping = { xmmsclient.PLAYBACK_STATUS_PLAY:  'play',
                    xmmsclient.PLAYBACK_STATUS_PAUSE: 'pause',
                    xmmsclient.PLAYBACK_STATUS_STOP:  'stop'
                  }
        state = result.value()

        if mapping.has_key(state):
            self._state = mapping[state]
            if self._state == 'stop': self.__id = None
        else:
            self._state = ''
            c.debug("Unrecognized state {0}".format(state))

        self.cg.update(STATE, self.get_state())
        #c.debug("new status : {0}".format(self._state))


    def __playlist_current_pos_cb(self, result):
        """
        Check if it's possible to go previous or next from the current
        position within the playlist.
        """

        self.__pos = result.value()
        self.__async.playlist_list_entries(None,
                self.__playlist_list_entries_cb)


    def __playlist_list_entries_cb(self, result):
        """
        Given the bounds of the current playlist, figure out whether going
        previous or next is possible.
        """

        self._count = len(result.value())
        #c.debug("position is {0}; playlist length is {1}".format(
        #       self.__pos, count))


    def __playback_current_id_cb(self, result):
        """
        Fetch metadata for the song that's now playing.
        """

        id = result.value()

        if id == 0:
            self.__id = None
            return

        self.__id = result.value()
        self.__async.medialib_get_info(self.__id, self.__medialib_get_info_cb)


    def __medialib_entry_changed_cb(self, result):
        """
        If the current song's metadata change, fetch it again.
        """

        if self.__id == result.value():
            self.__async.medialib_get_info(self.__id,
                    self.__medialib_get_info_cb)

    def __volume_changed_cb(self, result):
        self._volume = result.value()

    def __medialib_get_info_cb(self, result):
        """
        Update the cached metadata with the new information.
        """

        info = result.value()
        metadata = {}

        #c.debug("DEBUG: " + str(info))

        if info is not None:
            if info.has_key("url"):
                # don't use unquote_plus here since it can cause troubles if
                # the filename contains the '+' signs in it
                metadata["file"] = info["url"]
                metadata['file'] = unquote(info['url'].replace('+', ' '))
                #c.debug("URL: {0}".format(info['url']))

            if info.has_key("title"):
                metadata["title"] = info["title"]

            if info.has_key("artist"):
                metadata["artist"] = info["artist"]

            if info.has_key("album"):
                metadata["album"] = info["album"]
            elif info.has_key("channel"):
                metadata["album"] = info["channel"]

            if info.has_key("duration"):
                metadata["mtime"] = info["duration"]
                metadata["time"] = info["duration"] // 1000

            if info.has_key("genre"):
                metadata["genre"] = info["genre"]

            if info.has_key("comment"):
                metadata["comment"] = info["comment"]

            if info.has_key("rating"):
                metadata["rating"] = info["rating"]
            elif info.has_key("vote_score") and info.has_key("vote_count"):
                metadata["rating"] = int(info["vote_score"] //
                                         info["vote_count"])

            for key in ["album_front_large",
                        "album_front_small",
                        "album_front_thumbnail"]:
                if info.has_key(key):
                    metadata["arturl"] = info[key]
                    break

            if info.has_key("bitrate"):
                metadata["audio-bitrate"] = info["bitrate"]

            if info.has_key("samplerate"):
                metadata["audio-samplerate"] = info["samplerate"]

            if info.has_key("date"):
                metadata["date"] = info["date"]

            if info.has_key("track"):
                metadata["track"] = info["track"]

        vals = ('title','album','artist', 'genre', 'date', 'track', 'file',
               'rating', 'time')
        for key in vals:
            if not metadata.has_key(key):
                metadata[key] = ''

        if metadata['rating'] == '': metadata['rating'] = 0

        #has_song = (len(metadata) > 0)

        self._track = Track(
            title       = metadata['title'],
            album       = metadata['album'],
            artist      = metadata['artist'],
            rating      = metadata['rating'],
            genre       = metadata['genre'],
            tracknumber = metadata['track'],
            year        = metadata['date'],
            location    = metadata['file'],
            length      = metadata['time']
        )

        self.cg.update(TRACK, self._track)
        #c.debug("metada update : {0}".format(metadata))
        #c.debug("data update : {0}".format(data))
        #c.debug("track update : {0}".format(self._track.title))

    def __playback_playtime_cb(self, result):
        """
        Update with the latest position within the current song.
        """

        self._position = result.value() // 1000
        self.cg.update(TIME, self._position)
