# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import logging

from urllib import unquote

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import COVER, STATE, TIME, TRACK
from core.common import ABS_PATH

from Player import Player

c = logging.getLogger(__name__)

class Mpris(Player):
    DBUS_OBJECT = '/Player'

    capabilities = ('time', 'seek', 'volume')

    _player = None
    _signals = []

    def connect(self):
        """
            Connects player
            @return: boolean, True if successful, False if not
        """
        try:
            if not self.connected:
                proxy = dbus.SessionBus().get_object(self.DBUS_NAME,
                    self.DBUS_OBJECT)
                self._player = dbus.Interface(proxy,
                    'org.freedesktop.MediaPlayer')

                self._signals.append(self._player.connect_to_signal(
                    'StatusChange', self.status_changed))
                self._signals.append(self._player.connect_to_signal(
                    'TrackChange', self.track_changed))
                self.connected = True
        except:
            self.disconnect()
        return self.connected

    def disconnect(self):
        """
            Disconnects player
        """
        for signal in self._signals:
            signal.remove()

        self._player = None
        self._signals = []
        self.connected = False

    def get_state(self, data=None):
        """
            Returns players actual state.
            @return: string - playing, paused, stopped
        """
        if not data:
            data = self._player.GetStatus()
        return data[0] + 1

    def get_track(self, data=None):
        """
            Prepares a Track object with current playing song data

            @param data: dictionary, contains info about current playing song.
                optional
            @return: Track, object with current playing song data
        """
        if not data:
            data = self._player.GetMetadata()

        vals = ('title','album','artist', 'genre', 'year', 'tracknumber',
                'location', 'arturl', 'rating')
        for key in [key for key in vals if key not in data]:
            c.warning('%s: %s not found in current track' % (self.__class__.__name__, key))
            data[key] = ''

        track = Track(
            title       = data['title'],
            album       = data['album'],
            artist      = data['artist'],
            genre       = data['genre'],
            year        = data['year'],
            tracknumber = data['tracknumber'],
            rating = data['rating'],
            cover       = unquote(data['arturl'].encode('utf-8')),
            location    = unquote(data['location'].encode('utf-8'))
        )
        try:
            track.length = int(data['time'])
        except:
            try:
                track.length = int(data['mtime']) / 1000
            except:
                pass

        if track.cover.startswith('file://'):
            track.cover = track.cover[7:]
        return track

    def get_position(self):
        try:
            return self._player.PositionGet() / 1000
        except:
            return 0

    def set_position(self, time):
        if self.connected:
            self._player.PositionSet(time*1000)

    def get_volume(self):
        try:
            return self._player.VolumeGet()
        except:
            return 0

    def set_volume(self, volume):
        if self.connected:
            self._player.VolumeSet(volume)

# --------------------------------- CONTROLS --------------------------------- #

    def previous(self):
        """
            Changes song to previous
        """
        if self.connected:
            self._player.Prev()

    def play(self):
        """
            Plays or pauses the player
        """
        if self.connected:
            self._player.Pause()

    def next(self):
        """
            Changes song to next
        """
        if self.connected:
            self._player.Next()

# --------------------------------- HANDLERS --------------------------------- #

    def status_changed(self, message):
        self.cg.update(STATE, self.get_state(message))

    def track_changed(self, message):
        self.cg.update(TRACK, self.get_track(message))
