# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import logging

from urllib import unquote

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import COVER, STATE, TIME, TRACK
from core.common import ABS_PATH

from Player import Player

c = logging.getLogger(__name__)

class Mpris2(Player):
    DBUS_OBJECT = '/org/mpris/MediaPlayer2'

    capabilities = ('time', 'seek', 'volume')

    _player = None
    _properties = None
    _signals = []
    _track_id = None

    def connect(self):
        """Connects player
           @return: boolean, True if successful, False if not
        """
        try:
            if not self.connected:
                proxy = dbus.SessionBus().get_object(self.DBUS_NAME,
                    self.DBUS_OBJECT)
                self._player = dbus.Interface(proxy,
                    'org.mpris.MediaPlayer2.Player')
                self._properties = dbus.Interface(proxy,
                    'org.freedesktop.DBus.Properties')
                self._signals.append(self._properties.connect_to_signal(
                    'PropertiesChanged', self.properties_changed))
                self.connected = True
        except:
            self.disconnect()
        return self.connected

    def disconnect(self):
        """Disconnects player"""
        for signal in self._signals:
            signal.remove()

        self._player = None
        self._properties = None
        self._signals = []
        self.connected = False

    def get_state(self, status=None):
        """
            Returns players actual state.
            @return: string - playing, paused, stopped
        """
        if not status:
            status = self._properties.Get('org.mpris.MediaPlayer2.Player', 'PlaybackStatus')
        if status == 'Playing':
            return State.PLAYING
        elif status == 'Paused':
            return State.PAUSED
        elif status == 'Stopped':
            return State.STOPPED

    def get_track(self, data=None):
        if not data:
            data = self._properties.Get('org.mpris.MediaPlayer2.Player', 'Metadata')

        try:
            self._track_id = data['mpris:trackid']
        except:
            self._track_id = None

        vals = {}

        try:
            vals['title'] = data['xesam:title']
        except:
            pass

        try:
            vals['album'] = data['xesam:album']
        except:
            pass

        try:
            vals['artist'] = ', '.join(data['xesam:artist'])
        except:
            pass

        try:
            vals['genre'] = data['xesam:genre'][0]
        except:
            pass

        try:
            vals['year'] = data['xesam:year'][0:4]
        except:
            pass

        try:
            vals['tracknumber'] = int(data['xesam:trackNumber'])
        except:
            pass

        try:
            vals['rating'] = int(data['xesam:userRating'])
        except:
            pass

        try:
            cover = unquote(data['mpris:artUrl'].encode('utf-8'))
            vals['cover'] = cover[7:] if cover.startswith('file://') else cover
        except:
            pass

        try:
            vals['location'] = unquote(data['xesam:url'].encode('utf-8'))
        except:
            pass

        try:
            vals['length'] = int(data['mpris:length'])/1000000
        except:
            pass

        return Track(**vals)

    def get_position(self):
        try:
            return self._properties.Get('org.mpris.MediaPlayer2.Player', 'Position') / 1000000
        except:
            return 0

    def set_position(self, time):
        if self.connected and self._track_id:
            self._player.SetPosition(self._track_id, time*1000000)

    def get_volume(self):
        try:
            return self._properties.Get('org.mpris.MediaPlayer2.Player', 'Volume') * 100
        except:
            return 0

    def set_volume(self, volume):
        if self.connected:
            return self._properties.Set('org.mpris.MediaPlayer2.Player', 'Volume', volume / 100)

# --------------------------------- CONTROLS --------------------------------- #

    def previous(self):
        """
            Changes song to previous
        """
        if self.connected:
            self._player.Previous()

    def play(self):
        """
            Plays or pauses the player
        """
        if self.connected:
            self._player.PlayPause()

    def next(self):
        """
            Changes song to next
        """
        if self.connected:
            self._player.Next()

# --------------------------------- HANDLERS --------------------------------- #

    def properties_changed(self, interface, changed, invalidated):
        if interface == 'org.mpris.MediaPlayer2.Player':
            if 'Metadata' in changed:
                self.cg.update(TRACK, self.get_track(changed['Metadata']))
            if 'PlaybackStatus' in changed:
                self.cg.update(STATE, self.get_state(changed['PlaybackStatus']))
            # what to do with invalidated ??
