# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import logging

from gettext import gettext as _

from core.track import State
from core.common import STATE

c = logging.getLogger(__name__)

class Player(object):
    """ This class has the basic functions for the player,
        There are some of them that must be implemented for each SpecificPlayer,
        Please, take a look at BansheePlayer.py for an example.
        All Players must extend this class.
    """

    __title__ = 'GenericPlayer'
    __version__ = '0.3'
    __authors__ = 'BadChoice (guitarboy000@gmail.com), Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = 'A Generic API to a Music Player. All Players must extend this'

    connected = False

    DBUS_NAME = None
    DBUS_OBJECT = None

    capabilities = ()

    cg = None

    _signal = None

    def __init__(self, cg):
        """
            Initializes class, begins dbus listening if DBUS_NAME is provided
            @param d: dbus.Interface
        """
        super(Player, self).__init__()
        self.cg = cg
        if self.DBUS_NAME:
            self._signal = dbus.SessionBus().add_signal_receiver(
                self._dbus_handler, "NameOwnerChanged",
                'org.freedesktop.DBus', None, '/org/freedesktop/DBus')

        if self.is_active():
            c.debug("Connecting player")
            if not self.connect():
                c.warning(_('Connection failed'))

    def is_active(self):
        """ @return: boolean, True if player is running, False if not
        """
        if self.DBUS_NAME in self.cg.dbus_iface.ListNames():
            return True
        else:
            return False

    def _dbus_handler(self, name, a=None, b=None):
        """Callback recieves dbus signal NameOwnerChanged, controls senders
           name and if required calls state_handler with opropriate state.
        """
        if name == self.DBUS_NAME:
            if self.is_active():
                if self.connect():
                    self.cg.update(STATE, State.STARTED)
            else:
                self.disconnect()
                self.cg.update(STATE, State.CLOSED)

    def close(self):
        """
            Disconnects player and stops listening dbus for NameOwnerChanged
            signal.
        """
        self.disconnect()
        if self._signal:
            self._signal.remove()
            self._signal = None

    def get_player_name(self):
        """
            Return players name
        """
        return self.__title__

    def get_player_description(self):
        """
            Return players description
        """
        return self.__desc__

    def get_player_version(self):
        """
            Return players version
        """
        return self.__version__

    def get_player_authors(self):
        """
            Return players authors
        """
        return self.__author__

# ------------------------------ ABSTRACT PART ------------------------------- #

    def connect(self):
        """
            Connects player
            @return: boolean, True if successful, False if not
        """
        raise NotImplementedError

    def disconnect(self):
        """
            Disconnects player
        """
        raise NotImplementedError

    def get_state(self):
        """
            Returns players actual state.
            @return: string - playing, paused, stopped
        """
        raise NotImplementedError

    def get_track(self):
        """
            Prepares a Track object with current playing song data

            @param data: dictionary, contains info about current playing song.
                optional
            @return: Track, object with current playing song data
        """
        raise NotImplementedError

    def previous(self):
        """
            Changes song to previous
        """
        pass

    def play(self):
        """
            Plays or pauses the player
        """
        pass

    def next(self):
        """
            Changes song to next
        """
        pass

    def mute(self):
        pass

    def set_rating(self, rating):
        pass

    def get_position(self):
        pass

    def set_position(self, time):
        pass

    def get_volume(self):
        pass

    def set_volume(self, volume):
        pass
