# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gobject
import logging
import os

from gtk.gdk import pixbuf_new_from_file_at_scale
from gtk.gdk import pixbuf_get_file_info

from widgets import GenericWidget

c = logging.getLogger(__name__)

class Button(GenericWidget):
    __gsignals__ = {
        'clicked' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
    }

    _normal = None
    _hover = None
    _pressed = None
    _current = None

    _normal_path = None
    _hover_path = None
    _pressed_path = None

    _hovered = False

    def __init__(self, size, normal, hover, pressed):
        super(Button, self).__init__(size)

        # set paths
        if normal and os.path.exists(normal):
            self._normal_path = normal
        elif normal:
            c.error("Button: file not found: " + normal)
        if hover and os.path.exists(hover):
            self._hover_path = hover
        elif hover:
            c.error("Button: file not found: " + hover)
        if pressed and os.path.exists(pressed):
            self._pressed_path = pressed
        elif pressed:
            c.error("Button: file not found: " + pressed)

        # set width and/or height if not defined
        if not self._size.width or not self._size.height:
            for path in (self._normal_path, self._hover_path, self._pressed_path):
                if path:
                    info, w, h = pixbuf_get_file_info(path)
                    if not self._size.width:
                        self._size.set_width(w)
                    if not self._size.height:
                        self._size.set_height(h)
                    break

        self._load()

    def scale(self, percent):
        self._size.scale(percent)
        if self._reflect:
            self._reflect.scale(percent)
        self._load()

    def _load(self):
        if self._normal_path:
            self._normal = pixbuf_new_from_file_at_scale(self._normal_path,
                                                         self._size.width,
                                                         self._size.height,
                                                         False)
        if self._hover_path:
            self._hover = pixbuf_new_from_file_at_scale(self._hover_path,
                                                        self._size.width,
                                                        self._size.height,
                                                        False)
        if self._pressed_path:
            self._pressed = pixbuf_new_from_file_at_scale(self._pressed_path,
                                                          self._size.width,
                                                          self._size.height,
                                                          False)
        self._current = self._normal
        self.set_size_request(self._size.width, self._size.height)
        self.queue_draw()

    def get_height(self):
        if self._reflect:
            return self._size.height + self._reflect.gap + self._reflect.height
        else:
            return self._size.height

    def do_button_press_event(self, event):
        if event.button == 1 and self._pressed:
            self._current = self._pressed
            self.queue_draw()
        return True

    def do_button_release_event(self, event):
        if event.button == 1:
            if self._pressed:
                self._current = self._normal
                self.queue_draw()
            if event.x <= self._size.width and event.y <= self._size.height:
                self.emit('clicked')
        return True

    def do_enter_notify_event(self, event):
        if self._hover:
            self._current = self._hover
            self._hovered = True
            self.queue_draw()
        return True

    def do_leave_notify_event(self, event):
        self._hovered = False
        if self._hover:
            self._current = self._normal
            self.queue_draw()
        return True

    def do_expose_event(self, event):
        if self._current:
            cr = event.window.cairo_create()
            cr.rectangle(event.area.x, event.area.y,
                         event.area.width, event.area.height)
            cr.clip()
            cr.translate(self._size.x, self._size.y)
            cr.set_source_pixbuf(self._current, 0, 0)
            cr.paint()

            # draw reflection
            if self._reflect:
                cr.translate(0, 2 * self._size.height + self._reflect.gap)
                cr.scale(1, -1)
                linear = cairo.LinearGradient(0,self._size.height,
                                              0,self._size.height-self._reflect.height)
                linear.add_color_stop_rgba(0, 0, 0, 0, self._reflect.alpha)
                linear.add_color_stop_rgba(1, 0, 0, 0, 0)
                cr.set_source_pixbuf(self._current, 0, 0)
                cr.mask(linear)

        return True

# ============================================================================ #

gobject.type_register(Button)

# ============================================================================ #

class StateButton(Button):
    def __init__(self, size):
        self._state = None
        self._states = {}
        self._paths = {}
        super(StateButton, self).__init__(size, None, None, None)

    def add_state(self, state, normal, hover, pressed):
        # set paths
        if normal and not os.path.exists(normal):
            c.error("StateButton: file not found: " + normal)
            normal = None
        if hover and not os.path.exists(hover):
            c.error("StateButton: file not found: " + hover)
            hover = None
        if pressed and not os.path.exists(pressed):
            c.error("StateButton: file not found: " + pressed)
            pressed = None

        if normal or hover or pressed:
            self._paths[state] = (normal, hover, pressed)
            if self._state is None:
                self._state = state
                # set width and/or height if not defined
                if not self._size.width or not self._size.height:
                    for path in self._paths[state]:
                        if path:
                            info, w, h = pixbuf_get_file_info(path)
                            if not self._size.width:
                                self._size.set_width(w)
                            if not self._size.height:
                                self._size.set_height(h)
                            self.set_size_request(self._size.width, self._size.height)
                            break
            self._states[state] = self._load_images(self._paths[state])

    def _load(self):
        for state, paths in self._paths.iteritems():
            imgs = self._load_images(paths)
            self._states[state] = imgs
            if self._state == state:
                self._normal, self._hover, self._pressed = imgs
                self._current = self._normal
        self.set_size_request(self._size.width, self._size.height)
        self.queue_draw()

    def _load_images(self, paths):
        imgs = [None, None, None]
        if paths[0]:
            imgs[0] = pixbuf_new_from_file_at_scale(paths[0],
                                                    self._size.width,
                                                    self._size.height,
                                                    False)
        if paths[1]:
            imgs[1] = pixbuf_new_from_file_at_scale(paths[1],
                                                    self._size.width,
                                                    self._size.height,
                                                    False)
        if paths[2]:
            imgs[2] = pixbuf_new_from_file_at_scale(paths[2],
                                                    self._size.width,
                                                    self._size.height,
                                                    False)
        return imgs

    def set_state(self, state):
        if state in self._states and self._state != state:
            self._state = state
            self._normal, self._hover, self._pressed = self._states[state]
            if self._hovered and self._hover:
                self._current = self._hover
            else:
                self._current = self._normal
            self.queue_draw()

# ============================================================================ #

gobject.type_register(StateButton)

# ============================================================================ #
