# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import cairo
import gobject
import gtk
import logging
import os

from gtk.gdk import pixbuf_new_from_file_at_scale
from gtk.gdk import pixbuf_get_file_info

from gettext import gettext as _

from widgets import GenericWidget

c = logging.getLogger(__name__)

class Image(GenericWidget):
    _path = None
    _surface = None

    _mask = None
    _mask_path = None

    _opacity = 1.0

    _radius = 0
    _src_radius = 0

    def __init__(self, size):
        super(Image, self).__init__(size)

    def get_height(self):
        if self._reflect:
            return self._size.height + self._reflect.gap + self._reflect.height
        else:
            return self._size.height

    def set_image(self, path):
        if self._path != path:
            if path and not os.path.exists(path):
                c.error('Image: file not found: ' + path)
                path = None
            self._path = path
            if not self._size.width or not self._size.height:
                info, w, h = pixbuf_get_file_info(path)
                if not self._size.width:
                    self._size.set_width(w)
                if not self._size.height:
                    self._size.set_height(h)
            self._load()

    def set_mask(self, path):
        if not os.path.exists(path):
            c.error('Cover: file not found ' + path)
            path = None
        if not path:
            self._mask_path = None
            self._mask = None
        elif path != self._mask_path:
            self._mask_path = path
            self._mask = pixbuf_new_from_file_at_scale(self._mask_path,
                                                       self._size.width,
                                                       self._size.height,
                                                       False)

    def set_opacity(self, opacity):
        if opacity >= 0 and opacity <= 1:
            self._opacity = opacity
        else:
            raise ValueError

    def set_round(self, radius):
        if radius > 0:
            if radius > (self._size.width/2) or radius > (self._size.height/2):
                return
            self._radius = radius
            self._src_radius = radius

    def scale(self, percent):
        self._size.scale(percent)
        if self._reflect:
            self._reflect.scale(percent)
        self.set_size_request(self.get_width(), self.get_height())
        self._load()

    def _load(self):
        if self._path:
            try:
                pixbuf = pixbuf_new_from_file_at_scale(self._path,
                                                   self._size.width,
                                                   self._size.height,
                                                   False)
                self._draw_surface(pixbuf)
            except:
                c.warning(_("Failed to load image") + ": " + self._path)
                self._path = None
                self._surface = None
        else:
            self._surface = None

        self.queue_draw()

    def _draw_surface(self, pixbuf):
        self._surface = cairo.ImageSurface(cairo.FORMAT_ARGB32,
                                           self._size.width,
                                           self._size.height)
        cr = gtk.gdk.CairoContext(cairo.Context(self._surface))
        cr.set_source_pixbuf(pixbuf, 0, 0)
        if self._radius:
            self._clip_round(cr)
        cr.paint_with_alpha(self._opacity)

        if self._mask:
            cr.set_operator(cairo.OPERATOR_DEST_IN)
            cr.set_source_pixbuf(self._mask, 0, 0)
            cr.paint()

    def _clip_round(self, cr):
        w = self._size.width
        h = self._size.height
        r = self._radius

        cr.move_to ( r, 0 )
        cr.line_to ( w-r, 0 )
        cr.curve_to( w-r, 0, w, 0, w, r )
        cr.line_to ( w, h-r )
        cr.curve_to( w, h-r, w, h, w-r, h )
        cr.line_to ( r, h )
        cr.curve_to( r, h, 0, h, 0, h-r )
        cr.line_to ( 0, r )
        cr.curve_to( 0, r, 0, 0, r, 0)
        cr.clip    ( )

    def do_expose_event(self, event):
        if self._surface:
            cr = event.window.cairo_create()
            cr.rectangle(event.area.x, event.area.y,
                         event.area.width, event.area.height)
            cr.clip()
            cr.translate(self._size.x, self._size.y)
            cr.set_source_surface(self._surface, self._size.x, self._size.y)
            cr.set_source_surface(self._surface, 0, 0)
            cr.paint()

            # draw reflection
            if self._reflect:
                cr.translate(0, 2 * self._size.height + self._reflect.gap)
                cr.scale(1, -1)
                linear = cairo.LinearGradient(0,self._size.height,
                                              0,self._size.height-self._reflect.height)
                linear.add_color_stop_rgba(0, 0, 0, 0,
                                           self._reflect.alpha * self._opacity)
                linear.add_color_stop_rgba(1, 0, 0, 0, 0)
                cr.set_source_surface(self._surface, 0, 0)
                cr.mask(linear)
        return True

# ============================================================================ #

gobject.type_register(Image)

# ============================================================================ #
