# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gtk
import pango

from gettext import gettext as _

PLUGINS_COL_NAME = 0
PLUGINS_COL_DESCRIPTION = 1
PLUGINS_COL_ENABLED = 2
PLUGINS_COL_CLASS = 3
PLUGINS_COL_TOTAL = 4

class PluginView(gtk.VBox):
    def __init__(self):
        super(PluginView, self).__init__(spacing=5)
        self._init_tree()
        self._init_buttons()

    def _init_tree(self):
        self._store = gtk.ListStore(str, str, bool, object)
        self._tree = gtk.TreeView(self._store)
        self._tree.set_reorderable(True)
        self._tree.set_headers_visible(False)
        self._tree.set_rules_hint(True)
        self._tree.connect('cursor-changed', self.on_cursor_changed)

        # plugin toggle column
        renderer = gtk.CellRendererToggle()
        renderer.set_property('xpad', 6)
        renderer.connect('toggled', self.on_plugin_toggle)
        column = gtk.TreeViewColumn(None, renderer, active=PLUGINS_COL_ENABLED)
        self._tree.append_column(column)

        # plugin name column
        renderer = gtk.CellRendererText()
        renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
        column = gtk.TreeViewColumn(None, renderer)
        column.set_cell_data_func(renderer, self.plugin_markup)
        self._tree.append_column(column)

        #self.pack_start(self._tree)
        sw = gtk.ScrolledWindow()
        #sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(self._tree)

        self.pack_start(sw, True, True, 0)

    def _init_buttons(self):
        box = gtk.HBox()

        self._about = gtk.Button(_('About Plugin'))
        self._config = gtk.Button(_('Configure Plugin'))

        self._about.set_sensitive(False)
        self._config.set_sensitive(False)

        self._about.connect('clicked', self.on_plugin_about)
        self._config.connect('clicked', self.on_plugin_config)

        box.pack_end(self._config, False, False)
        box.pack_end(self._about, False, False)

        self.pack_end(box, False, False)

    def on_cursor_changed(self, widget):
        model, it = widget.get_selection().get_selected()
        if it:
            self._about.set_sensitive(True)
            klass = model.get_value(it, PLUGINS_COL_CLASS)
            if hasattr(klass, 'configure'):
                self._config.set_sensitive(True)
            else:
                self._config.set_sensitive(False)
        else:
            self._about.set_sensitive(False)
            self._config.set_sensitive(False)

    def on_plugin_toggle(self, widget, path):
        it = self._store.get_iter(path)
        enabled = not self._store.get_value(it, PLUGINS_COL_ENABLED)
        self._store.set_value(it, PLUGINS_COL_ENABLED, enabled)

    def on_plugin_about(self, widget):
        model, it = self._tree.get_selection().get_selected()
        klass = model.get_value(it, PLUGINS_COL_CLASS)
        dialog = gtk.AboutDialog()
        dialog.set_logo_icon_name("covergloobus")
        dialog.set_name(model.get_value(it, PLUGINS_COL_NAME))
        dialog.set_version(klass.__version__)
        dialog.set_comments(model.get_value(it, PLUGINS_COL_DESCRIPTION))

        try:
            dialog.set_authors([a.strip() for a in klass.__authors__.split(',')])
        except:
            pass

        dialog.run()
        dialog.destroy()

    def on_plugin_config(self, widget):
        model, it = self._tree.get_selection().get_selected()
        klass = model.get_value(it, PLUGINS_COL_CLASS)
        dialog = gtk.Dialog(_('%s configuration') % klass.__title__,
                            self.get_toplevel(),
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                            gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))

        klass.configure(dialog)
        dialog.destroy()

    def plugin_markup(self, column, cell, store, it):
        name = store.get_value(it, PLUGINS_COL_NAME)
        desc = store.get_value(it, PLUGINS_COL_DESCRIPTION)
        cell.set_property('markup', "<b>%s</b>\n%s" % (name, desc))

    def set_plugins(self, plugins):
        self._store.clear()
        for plugin in plugins:
            self._store.append(plugin)

    def get_plugins(self):
        plugins = []
        it = self._store.get_iter_first()
        while it:
            plugin = []
            for i in xrange(PLUGINS_COL_TOTAL):
                plugin.append(self._store.get_value(it, i))
            it = self._store.iter_next(it)
            plugins.append(plugin)
        return plugins
