# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gobject
import logging
import os

from gtk.gdk import pixbuf_new_from_file_at_scale
from gtk.gdk import pixbuf_get_file_info

from gettext import gettext as _

from widgets import GenericWidget

c = logging.getLogger(__name__)

class Slider(GenericWidget):
    __gsignals__ = {
        'value-changed' : (gobject.SIGNAL_RUN_LAST,
                           gobject.TYPE_NONE, (int,)),
    }

    _val = 0
    _max = 1

    _background = None
    _foreground = None
    _thumb = None

    _background_path = None
    _foreground_path = None

    _thumb_path = None
    _thumb_size = None

    _top = 0
    _bottom = 0
    _left = 0
    _right = 0


    size = None

    def __init__(self, size, background, foreground):
        super(Slider, self).__init__(size)
        if background:
            if os.path.exists(background):
                self._background_path = background
            else:
                c.error('Seekbar: file not found: ' + background)

        if foreground:
            if os.path.exists(foreground):
                self._foreground_path = foreground
            else:
                c.error('Seekbar: file not found: ' + foreground)

        self._load()

    def get_x(self):
        if self._left < 0:
            return self._size.x + self._left
        return self._size.x

    def get_y(self):
        if self._top < 0:
            return self._size.y + self._top
        return self._size.y

    def _load(self):
        if self._background_path:
            self._background = pixbuf_new_from_file_at_scale(self._background_path,
                                                             self._size.width,
                                                             self._size.height,
                                                             False)
        if self._foreground_path:
            self._foreground = pixbuf_new_from_file_at_scale(self._foreground_path,
                                                             self._size.width,
                                                             self._size.height,
                                                             False)

    def _load_thumb(self):
        self._thumb = pixbuf_new_from_file_at_scale(self._thumb_path,
                                                    self._thumb_size.width,
                                                    self._thumb_size.height,
                                                    False)

    def set_max(self, m):
        if m > 1:
            self._max = float(m)
            self._val = 0.0
            self.queue_draw()
        else:
            self._max = 1

    def set_value(self, val):
        self._val = min(val, self._max) if val >= 0 else 0
        self.queue_draw()

    def set_thumb(self, thumb, size, top, bottom, left, right):
        self._top, self._bottom, self._left, self._right = top, bottom, left, right
        self._thumb_path = thumb
        self._thumb_size = size
        self._load_thumb()
        self.set_size_request(self.get_width(), self.get_height())

    def scale(self, percent):
        self._size.scale(percent)
        self._load()
        if self._thumb_size:
            self._thumb_size.scale(percent)
            self._load_thumb()
        self.set_size_request(self.get_width(), self.get_height())
        self.queue_draw()

    def do_button_press_event(self, event):
        return True

    def do_expose_event(self, event):
        raise NotImplementedError

    def do_button_release_event(self, event):
        raise NotImplementedError

# ============================================================================ #

class HSlider(Slider):
    def get_width(self):
        width = self._size.width
        if self._left < 0:
            width += -self._left
        if self._right < 0:
            width += -self._right
        return width

    def get_height(self):
        height = self._size.height
        if self._top < 0:
            height += -self._top
        if self._thumb_size and height < self._thumb_size.height:
            height = self._thumb_size.height
        return height

    def do_expose_event(self, event):
        width = int(self._val * self._size.width / self._max)
        cr = event.window.cairo_create()
        cr.rectangle(event.area.x, event.area.y,
                     event.area.width, event.area.height)
        cr.clip()
        cr.save()
        cr.translate(self._size.x, self._size.y)
        if self._background:
            cr.set_source_pixbuf(self._background, 0, 0)
            cr.paint()
        if self._foreground:
            cr.rectangle(0, 0, width, self._size.height)
            cr.clip()
            cr.set_source_pixbuf(self._foreground, 0, 0)
            cr.paint()

        if self._thumb:
            cr.restore()
            w = self._size.width - self._right - self._thumb_size.width - self._left
            x = self._size.x + self._left + int(self._val * w / self._max)
            cr.translate(x, self._size.y + self._top)
            cr.set_source_pixbuf(self._thumb, 0, 0)
            cr.paint()

    def do_button_release_event(self, event):
        if event.button == 1:
            x = event.x + self._left if self._left < 0 else event.x
            if x <= self._size.width:
                self.emit('value-changed', x * self._max / self._size.width)
        return True

# ============================================================================ #

gobject.type_register(HSlider)

# ============================================================================ #

class VSlider(Slider):
    def get_width(self):
        width = self._size.width
        if self._left < 0:
            width += -self._left
        if self._thumb_size and width < self._thumb_size.width:
            width = self._thumb_size.width
        return width

    def get_height(self):
        height = self._size.height
        if self._top < 0:
            height += -self._top
        if self._bottom < 0:
            height += -self._bottom
        return height

    def do_expose_event(self, event):
        height = int(self._val * self._size.height / self._max)
        cr = event.window.cairo_create()
        cr.rectangle(event.area.x, event.area.y,
                     event.area.width, event.area.height)
        cr.clip()
        cr.save()
        cr.translate(self._size.x, self._size.y)
        if self._background:
            cr.set_source_pixbuf(self._background, 0, 0)
            cr.paint()
        if self._foreground:
            cr.rectangle(0, self._size.height - height, self._size.width, height)
            cr.clip()
            cr.set_source_pixbuf(self._foreground, 0, 0)
            cr.paint()

        if self._thumb:
            cr.restore()
            h = self._size.height - self._thumb_size.height - self._bottom - self._top
            y = self._size.y + self._top + h - int(self._val * h / self._max)
            cr.translate(self._size.x + self._left, y)
            cr.set_source_pixbuf(self._thumb, 0, 0)
            cr.paint()

    def do_button_release_event(self, event):
        if event.button == 1:
            y = event.y + self._top if self._top < 0 else event.y
            y = self._size.height - y
            if 0 <= y <= self._size.height:
                self.emit('value-changed', y * self._max / self._size.height)
        return True

# ============================================================================ #

gobject.type_register(VSlider)

# ============================================================================ #
