# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gobject
import gtk
import os

from threading import Thread
from xml.dom import minidom

from widgets.themeinstallwindow import ThemeInstallWindow

from gettext import gettext as _
from core.common import ABS_PATH

THEMES_COL_NAME = 0
THEMES_COL_PIXBUF = 1
THEMES_COL_PATH = 2

THEME_SCREENSHOT_SIZE = 192

class ThemeView(gtk.VBox):
    __gsignals__ = {
        'theme-changed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (str, )),
    }

    _selected = None
    _themes = None
    _total = 0

    def __init__(self):
        super(ThemeView, self).__init__(spacing=5)
        self._init_view()
        self._init_buttons()

    def _init_view(self):
        self._store = gtk.ListStore(str, gtk.gdk.Pixbuf, str)
        self._store.set_sort_column_id(THEMES_COL_NAME, gtk.SORT_ASCENDING)

        self._view = gtk.IconView(self._store)
        self._view.set_selection_mode(gtk.SELECTION_SINGLE)
        self._view.set_text_column(THEMES_COL_NAME)
        self._view.set_pixbuf_column(THEMES_COL_PIXBUF)
        self._view.set_columns(1)
        self._view.connect('realize', self.on_realize)

        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(self._view)

        self.pack_start(sw)

        self._noscreenshot = gtk.gdk.pixbuf_new_from_file_at_size(ABS_PATH + \
                                                    '/themes/noscreenshot.png',
                                                    THEME_SCREENSHOT_SIZE,
                                                    THEME_SCREENSHOT_SIZE)

    def _init_buttons(self):
        box = gtk.HBox()

        install = gtk.Button(_('Install New Theme'))
        #edit = gtk.Button(_('Edit Theme'))

        #edit.set_sensitive(False)

        install.connect('clicked', self.on_theme_install)
        #edit.connect('clicked', self.on_theme_edit)

        box.pack_end(install, False, False)
        #box.pack_end(edit, False, False)

        self.pack_end(box, False, False)
        box.show_all()

    def on_selection_changed(self, widget):
        cursor = widget.get_cursor()
        if cursor:
            it = self._store.get_iter(cursor[0])
            path = self._store.get_value(it, THEMES_COL_PATH)
            self._selected = path
            self.emit('theme-changed', path)

    def set_themes(self, themes):
        self._store.clear()

        for path in themes:
            name = os.path.basename(path)
            self._store.append([name, self._noscreenshot, path])

    def set_selected(self, path):
        self._selected = path

    def get_selected(self):
        return self._selected

    def on_realize(self, widget):
        Thread(target=self._load_screenshots).start()
        self._view.connect('selection-changed', self.on_selection_changed)
        self._view.set_item_width(self._view.get_allocation().width - 32)
        self._progressbar = gtk.ProgressBar()
        self._progressbar.set_fraction(0.0)
        self._progressbar.set_text(_("Loading themes..."))
        self.pack_start(self._progressbar, False, False)
        self._progressbar.show()

    def _load_screenshots(self):
        it = self._store.get_iter_root()
        total = float(len(self._store))
        done = 0
        while it:
            done += 1
            path = self._store.get_value(it, THEMES_COL_PATH)
            if os.path.exists(path + "/screenshot.png"):
                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(path + \
                                                        "/screenshot.png",
                                                        THEME_SCREENSHOT_SIZE,
                                                        THEME_SCREENSHOT_SIZE)
                if pixbuf:
                    gobject.idle_add(self._set_screenshot, pixbuf, it,
                                     done / total)
            it = self._store.iter_next(it)

    def _set_screenshot(self, pixbuf, it, done):
        self._store.set_value(it, THEMES_COL_PIXBUF, pixbuf)
        if self._selected == self._store.get_value(it, THEMES_COL_PATH):
            select = self._store.get_path(it)
            self._view.select_path(select)
            self._view.scroll_to_path(select, True, 0.5, 0.5)
        if done < 1.0:
            self._progressbar.set_fraction(done)
        else:
            self._progressbar.destroy()
        return False

    def on_theme_install(self, widget):
        theme_window = ThemeInstallWindow()

    def on_theme_edit(self, widget):
        if self._selected:
            pass
            #TODO

gobject.type_register(ThemeView)
